// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#ifndef __RTPI_RTPI__
#define __RTPI_RTPI__

#ifdef WIN32
#  pragma warning (disable: 4786 4181)
#endif


#ifdef WIN32
#include <util/refcount.cpp>
#else
#include <util/refcount.cc>
#endif

#include <rtpi/rtpiPacket.h>
#include <rtpi/rtcpiPacket.h>
#include <rtpi/rtpihandler.h>
#include <rtpi/rtpiparticipant.h>
#include <rtpi/rtpiInterfaces.h>
#include <rtpi/rtpiTransport.h>
#include <rtpi/rtpiSeqNumber.h>
#include <util/types.h>
#include <util/Buffer.h>


#include <string>


namespace RTPI {

  typedef std::vector<RefCount<Buffer> > BufferVector;


  class Rtpi {
  public:
    Rtpi();
    virtual ~Rtpi();

    // group membership
    virtual RtpiParticipant* join_group(std::string username, RtcpiReportInterface* rtcpiReport, 
					RtpiFeedbackInterface* rtpiFeedback, Transport* rtpiTransport);
    virtual void leave_group();
    virtual bool group_joined();

    // manage participants
    virtual RtpiParticipant* get_own_participant();
    virtual RtpiParticipant* get_participant(uint32 pid);
    virtual bool participant_exists(uint32 pid);

    virtual RtpiParticipantList::iterator get_participants_begin();    // get the end iterator of the participant list
    virtual RtpiParticipantList::iterator get_participants_end();      // get next valid participant
    virtual RtpiParticipantList::iterator get_next_participant(RtpiParticipantList::iterator&);

    // manage subcomponents
    virtual void add_subcomponent(uint32 subid, std::string name = "", bool active = true);
    virtual void delete_subcomponent(uint32 subid);

    virtual RtpiSubComponent* get_subcomponent(uint32 subid);
    virtual bool subcomponent_exists(uint32 subid);
    
    virtual SubComponentMap::iterator get_subcomponents_begin();
    virtual SubComponentMap::iterator get_subcomponents_end();    
    virtual SubComponentMap::iterator get_next_subcomponent(SubComponentMap::iterator&);    

    virtual void activate_subcomponent(uint32 subid);
    virtual void deactivate_subcomponent(uint32 subid);
    virtual bool subcomponent_is_active(uint32 subid);
    virtual void deactivate_all_subcomponents();

    virtual void set_subcomponent_name(uint32 subid, std::string name);
    virtual std::string get_subcomponent_name(uint32 subid);
    virtual bool subcomponent_has_name(uint32 subid);

    // helper functions
    virtual void send_rtcpi_note(std::string note = "");
    virtual uint32 get_current_time();

    // outgoing data
    virtual RefCount<BufferVector> create_rtpi_packets(RefCount<RtpiAdu> adu);
    virtual RefCount<BufferVector> create_rtpi_packets(RefCount<RtpiAduVector> adus);

    // incoming data
    virtual RefCount<RtpiAduVector> create_rtpi_adus(RefCount<Buffer> buf);
    virtual void handle_received_rtpi(RefCount<RtpiAdu> adu, uint32 senderIpAddress);
    virtual void handle_received_rtpi(RefCount<RtpiAduVector> adus, uint32 senderIpAddress);
    virtual void handle_received_rtcpi(RefCount<Buffer> buf, uint32 senderIpAddress);

  protected:
    RtcpiReportInterface* rtcpiReport;
    RtpiFeedbackInterface* rtpiFeedback;

    RtcpiPacketFactory* rtcpiPacketFactory;
    RtpiPacketFactory* rtpiPacketFactory;
    Transport* transport;

    RtpiHandler* rtpiHandler;

    bool joined;
  };
}


#endif
