// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

/**********************************************************************

 program : rtpiException.h
 authors : Juergen Vogel
 date    : 05.04.2000
 desc    : defines Exception thrown by RTP/I

 **********************************************************************/


#ifndef __RTPI_EXCEPTION__
#define __RTPI_EXCEPTION__

#ifdef WIN32
#  pragma warning (disable: 4786)
#endif

#include <string>

namespace RTPI {
  class RtpiException {
  public:
    enum types {
      ERR_RTPI_SOCKET,
      ERR_RTPI_INTERNAL,
      ERR_RTPI_PARSE,
      ERR_RTPI_FLUSH,
      ERR_RTPI_PARSE_HDR,
      ERR_RTPI_FLUSH_HDR,
      ERR_RTPI_PARSE_PKT,
      ERR_RTPI_FLUSH_PKT,
      ERR_RTPI_PARSE_ADU,
      ERR_RTPI_FLUSH_ADU,
      ERR_RTPI_WRONG_VERSION,
      ERR_RTPI_PACKET_FORMAT,
      ERR_RTPI_ILLEGAL_SEQUENCE_NUMBER,
      ERR_RTCPI_PARSE,
      ERR_RTCPI_FLUSH,
      ERR_RTCPI_WRONG_VERSION,
      ERR_RTCPI_PACKET_FORMAT,
      ERR_PARTICIPANT_REJECTED,
      ERR_PARTICIPANT_INVALID,
      ERR_OWN_PARTICIPANT_IGNORED,
      ERR_PARTICIPANT_NOT_FOUND,
      ERR_PARTICIPANT_TO_BE_REMOVED,
      ERR_SUBCOMPONENT_INVALID
    };

    RtpiException(int _errCode, std::string _errMsg) : errCode(_errCode), errMsg(_errMsg) {}

    void Print();

    int errCode;
    std::string errMsg;
  };
}


#endif
