// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#ifndef _RtpiInterfaces_h_
#define _RtpiInterfaces_h_

#include <vector>
#include <map>
#include <string>

#include <util/types.h>
#include <rtpi/rtpiSeqNumber.h>

namespace RTPI {
  class RtpiHandler;
  class RtpiParticipant;
  class RtpiSubComponent;

  typedef std::pair<const uint32,RtpiSubComponent> SubComponentMapElement;

  // Delegate the act of sending an report to the appcliation
  // RtcpiReportInterface
  class RtcpiReportInterface {
  public:
    // send report to a socket
    virtual void send_rtcpi(RefCount<Buffer>) =0;
    // get infos about our host
    virtual u_int32 get_local_address(int i=0) = 0;
    virtual int addr_is_local(u_int32) = 0;
    // get infos about media
    virtual u_int32   get_session_bandwith() =0;
    // get a new unique pid
    virtual u_int32   get_new_pid() = 0;
    // get a new unique subid
    virtual u_int32   get_new_subid() = 0;
    // implementation of the following functions is optional.
    // provide a base timestamp. Needed if inter media
    // synchronization (via get_ts()) is used by application.
    virtual u_int32   get_session_time()  { return 0; }
    // they override the sdes data set for master participants
    virtual std::string    get_session_name(string ="") { return ""; }
    virtual std::string    get_session_email()  { return ""; }
    virtual std::string    get_session_phone()  { return ""; }
    virtual std::string    get_session_loc()  { return ""; }
    virtual std::string    get_session_note()  { return ""; }
    virtual std::string    get_session_tool()  { return ""; }
  };


  // the applications is informed about changes in the
  // subcomponents via this interface. The implementation
  // of this interface is optional.
  class RtpiFeedbackInterface {
  public:
    enum types {
      // type of ADU
      PKT_RTPI_EVENT = 0,
      PKT_RTPI_STATE = 1,
      PKT_RTPI_DELTA_STATE = 2,
      PKT_RTPI_STATE_QUERY = 3,
      PKT_RTPI_CUE = 4,
      PKT_RTCPI = 100
    };

    // the following functions provide feedback for the application
    // sub component was changed
    virtual void  subcomponent_changed(SubComponentMapElement* subcomp, uint8 reason) {  };
    // sub component was added by a rtpi (reaason = RTPI packet type 0 - 4)
    // or rtcpi (reason = 100) packet
    virtual void  subcomponent_added(SubComponentMapElement* subcomp, int reason) {  };
    // sub component was removed
    virtual void  subcomponent_removed(uint32 subid) {  };
    // one or more subcomponents are added/removed/changed
    // called once per change (which may affect multiple subcomponents)
    virtual void  subcomponent_db_modified() { };

    // non-master participant was added
    virtual void  participant_added(uint32 pid) {  };
    // non-master participant was removed
    virtual void  participant_removed(uint32 pid) {  };
    // remote participant changed sdes info
    virtual void  participant_changed(uint32 pid) {  };

    // the following function can be implemented to control the protocol stack
    // should a sub_component_name be reported
    virtual bool  report_subcomponent_name(uint32 subid) { return true; };
  };

};

#endif






