// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#include <rtpi/rtpiSeqNumber.h>

#define xDEBUG0
#define xDEBUG1
#define xDEBUG2
#define xDEBUG3
#define xDEBUG4
#define xDEBUG5
#define xDEBUG8

// range in which packets might differ from current sequence number
// without beeing considered as a sequence number jump
#define MAX_MISORDER 100

// we report a local subcomponent for which we don't have received a
// report from others since x rtcpi report intervals
#define SUBCOMPONENT_REPORT_INTERVAL 2

// we remove a remote subcomponent for which we don't have received a
// report from others since x rtcpi report intervals
#define SUBCOMPONENT_TIMEOUT_INTERVAL 6

// activate subcomponent if data is received
#define ACTIVATE_BY_DATA true


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
std::ostream& RTPI::operator<<(std::ostream& os, RTPI::ParticipantSeqMap& pm) {
  for (ParticipantSeqMap::iterator ppos = pm.begin(); ppos != pm.end(); ++ppos) {
    os << "Participant Sequence:  Pid:          \t" << ppos->first << std::endl;
    os << "   Initialized: \t" << ppos->second.initialized << std::endl;
    os << "   Last_seq: \t" << ppos->second.last_seq << std::endl;
    os << "   Base_seq: \t" << ppos->second.base_seq << std::endl;
    os << "   Seq_cycles: \t" << ppos->second.seq_cycles << std::endl;
    os << "----------------------------" << std::endl;
  }
  return os;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
std::ostream& RTPI::operator<<(std::ostream& os, RTPI::ADUTypeSeqMap& am) {
  for (ADUTypeSeqMap::iterator ppos = am.begin(); ppos != am.end(); ++ppos) {
    os << "ADU type:     \t";
    switch(ppos->first) {
    case RtpiHeader::PKT_EVENT:
      os << "Event" << std::endl;;
      break;
    case RtpiHeader::PKT_CUE:
      os << "Cue" << std::endl;;
      break;
    case RtpiHeader::PKT_STATE:
      os << "State" << std::endl;;
      break;
    case RtpiHeader::PKT_DELTA_STATE:
      os << "Delta State" << std::endl;;
      break;
    case RtpiHeader::PKT_STATE_QUERY:
      os << "State Query" << std::endl;;
      break;
    }
    os << ppos->second.participants;
    os << "**************************************************" << std::endl;
  }
  return os;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
std::ostream& RTPI::operator<<(std::ostream& os, RTPI::SubComponentMap& sm) {
  os << "Number of SubComponents " << sm.size() << std::endl;
  for (SubComponentMap::iterator ppos = sm.begin(); ppos != sm.end(); ++ppos) {
    os << "SubComponent:  SubID  \t" << ppos->first << " \t alt. SubID \t " << ppos->second.alt_subid << std::endl;
    os << "  name:       \t";
    for (int i = 0; i < ppos->second.name_length; i++) {
      os << (( ppos->second.name[i] > ' ' &&  ppos->second.name[i] < '~') ?  ppos->second.name[i] : '*');
    }
    os << std::endl;
    os << "  Local Active: \t" << ppos->second.local_active << std::endl;
    os << "  Global Active: \t" << ppos->second.global_active << std::endl;
    os << "  Reported Active: \t" << ppos->second.reported_active << std::endl;
    os << "  Local:       \t" << ppos->second.local << std::endl;
    os << "  Last Report: \t" << ppos->second.last_report.tv_sec << "." <<  ppos->second.last_report.tv_usec << std::endl;
    os << "  Reported by: \t" << ppos->second.reported_by << std::endl;
    os << "  Timeout Base: \t" << ppos->second.timeout_base.tv_sec << "." <<  ppos->second.timeout_base.tv_usec << std::endl;
    os << "  last pt: \t" << ppos->second.last_pt << std::endl;
    os << ppos->second.ADUtypes;
    os << "================================================================" << std::endl;
  }
  return os;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
std::ostream& RTPI::operator<<(std::ostream& os, RTPI::RtpiSubComponentSpace& sm) {
  return os << sm.subComponents << std::endl;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void RTPI::RtpiParticipantSeq::err_out(uint8 t) {
  switch (t) {
  case SEQ_OK: std::cout << "Ok."; break;
  case SEQ_DUPLICATE:  std::cout << "Duplicate sequence numbers."; break;
  case SEQ_GAP:  std::cout << "Gap in sequence numbers."; break;
  case SEQ_JUMP:  std::cout << "Sequence numbers made a large jump."; break;
  case SEQ_INITIALIZED:  std::cout << "New sequence number initalized."; break;
  }
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
uint8 RTPI::RtpiParticipantSeq::seq_ok(uint16 _seq) {
  #ifdef DEBUG2
  std::cout << "RtpiParticipantSeq::seq_ok(" << _seq << ")" << std::endl;
  #endif

  // do we have a new sequence number
  if (!initialized) {
    init_seq(_seq);
    last_seq = _seq;
    return RtpiParticipantSeq::SEQ_INITIALIZED;
  }

  // check existing number
  uint16 diff = _seq - last_seq;

  if (diff == 0) {
    return RtpiParticipantSeq::SEQ_DUPLICATE;
  }

  if (diff < MAX_MISORDER) {
    // does sequence number wrap around
    if (_seq < last_seq)
      seq_cycles++;
    last_seq = _seq;
    if (diff == 1)
      return RtpiParticipantSeq::SEQ_OK;
    else
      return RtpiParticipantSeq::SEQ_GAP;
  }

  diff = last_seq - _seq;
  if (diff < MAX_MISORDER) {
    return RtpiParticipantSeq::SEQ_DUPLICATE;
  }

  init_seq(_seq);
  last_seq = _seq;
  return RtpiParticipantSeq::SEQ_JUMP;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
uint16 RTPI::RtpiParticipantSeq::get_next_seq() {
  #ifdef DEBUG2
  std::cout << "RtpiParticipantSeq::get_next_seq()" << std::endl;
  #endif

  // do we have a new sequence number
  if (!initialized) {
    init_seq();
  }

  // check for seq_cycles
  if (last_seq > last_seq + 1)
    ++seq_cycles;

  return ++last_seq;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void RTPI::RtpiParticipantSeq::insert_gap(uint16 gap) {
  #ifdef DEBUG2
  std::cout << "RtpiParticipantSeq::insert_gap("<<gap<<")" << std::endl;
  #endif

  // do we have a new sequence number
  if (!initialized) {
    init_seq();
  }

  // check for seq_cycles
  if (last_seq > last_seq + gap)
    ++seq_cycles;

  last_seq += gap;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
uint8 RTPI::RtpiSubComponent::refresh_component() {
  #ifdef DEBUG2
  std::cout << "RtpiSubComponent(" << subid << "/" << alt_subid << ")::refresh_component()" << std::endl;
  #endif

  int ret = RtpiSubComponent::SUBCOMP_OK;
  if (!initialized) {
    initialized = true;
    ret = RtpiSubComponent::SUBCOMP_INITIALIZED;
  }

  // for timeout
  timeout_base = unixtime();

  return ret;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
uint8 RTPI::RtpiSubComponent::data_received(int _last_pt) {
  #ifdef DEBUG2
  std::cout << "RtpiSubComponent(" << subid << "/" << alt_subid << ")::data_received()" << std::endl;
  #endif

  int ret = refresh_component();

  // activate subcomponent that has not yet been reported?
  if (!timerisset(&last_report) && !global_active) {
    global_active = true;
    ret += RtpiSubComponent::SUBCOMP_CHANGED_ACTIVE;
  }

  // remember last pt for this component
  if (_last_pt != -1) {
    if ((last_pt != -1) && (last_pt != _last_pt)) {
      // payload type changed.
      #ifdef DEBUG2
      std::cout << "RtpiSubComponent(" << subid << "/" << alt_subid << ")::data_received(): Payload type changed for sub-component." << std::endl;
      #endif
      ret += RtpiSubComponent::SUBCOMP_PAYLOAD_CHANGED;
    }
    last_pt = _last_pt;
  }
  return ret;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
uint8 RTPI::RtpiSubComponent::data_sent(int _last_pt) {
  #ifdef DEBUG2
  std::cout << "RtpiSubComponent(" << subid << "/" << alt_subid << ")::data_sent()" << std::endl;
  #endif

  int ret = refresh_component();

  // we must set subcomp to local
  if (!local) {
    local = true;
    ret += RtpiSubComponent::SUBCOMP_SET_LOCAL;
  }

  // activate subcomponent?
  if (ACTIVATE_BY_DATA && !local_active) {
    local_active = true;
    ret += RtpiSubComponent::SUBCOMP_CHANGED_ACTIVE;
  }

  // remember last pt for this component
  if (_last_pt != -1) {
    if ((last_pt != -1) && (last_pt != _last_pt)) {
      // payload type changed.
      #ifdef DEBUG2
      std::cout << "RtpiSubComponent(" << subid << "/" << alt_subid << ")::data_received(): Payload type changed for sub-component." << std::endl;
      #endif
      ret += RtpiSubComponent::SUBCOMP_PAYLOAD_CHANGED;
    }
    last_pt = _last_pt;
  }
  return ret;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
uint8 RTPI::RtpiSubComponent::report_received(uint32 _pid, bool _active, char* _name, int _name_length) {
  #ifdef DEBUG2
  std::cout << "RtpiSubComponent(" << subid << "/" << alt_subid << ")::report_received(" << _pid << ", " << _active << ", **** , "
       << _name_length << ")" << std::endl;
  #endif

  // ignore passive reports from others for an locally active component
  if (local && local_active && !_active) {
    return RtpiSubComponent::SUBCOMP_NO_CHANGES;
  }

  int ret_value = refresh_component();

  // remember the time this report is received
  last_report = unixtime();
  reported_by = _pid;
  // clear this flag since we do not report
  reported_active = false;

  if (global_active != _active) {
    global_active = _active;
    ret_value += RtpiSubComponent::SUBCOMP_CHANGED_ACTIVE;
  }

  if ((_name_length != 0) &&
      ((name_length != _name_length) || (memcmp(_name, name, _name_length) != 0))) {
    memcpy(name, _name, _name_length);
    name_length = _name_length;
    ret_value += RtpiSubComponent::SUBCOMP_CHANGED_NAME;
  }

  return ret_value;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
uint8 RTPI::RtpiSubComponent::report_sent(uint32 _pid) {
  #ifdef DEBUG2
  std::cout << "RtpiSubComponent(" << subid << "/" << alt_subid << ")::report_sent(" << _pid << ")" << std::endl;
  #endif

  int ret_value = refresh_component();

  // remember the time this report is received
  last_report = unixtime();
  reported_by = _pid;
  // remember how we reported this component
  reported_active = local_active;

  return ret_value;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
uint8 RTPI::RtpiSubComponent::add_component(bool _active, char* _name, int _name_length) {
  #ifdef DEBUG2
  std::cout << "RtpiSubComponent("<< subid << "/" << alt_subid << ")::add_component(" << _active << ", **** , "
       << _name_length << ")" << std::endl;
  #endif

  // if we introduce a new subcomponent, we should wait only one interval!
  // to be implemented!
  int ret_value = refresh_component();

  local = true;
  if (local_active != _active) {
    local_active = _active;
    ret_value += RtpiSubComponent::SUBCOMP_CHANGED_ACTIVE;
  }

  if ((_name_length != 0) &&
      ((name_length != _name_length) || (memcmp(_name, name, _name_length) != 0))) {
    memcpy(name, _name, _name_length);
    name_length = _name_length;
    ret_value += RtpiSubComponent::SUBCOMP_CHANGED_NAME;
  }

  return ret_value;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void RTPI::RtpiSubComponent::release_component() {
  #ifdef DEBUG2
  std::cout << "RtpiSubComponent(" << subid << "/" << alt_subid << ")::release_component()" << std::endl;
  #endif

  initialized = true;
  local = false;
  local_active = false;
  reported_active = false;

  refresh_component();
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
bool RTPI::RtpiSubComponent::must_report(uint32 _pid, int _interval) {
  #ifdef DEBUG2
  std::cout << "RtpiSubComponent(" << subid << "/" << alt_subid << ")::must_report(" << _pid << ", " << _interval << ")" << std::endl;
  #endif

  // we never report subcomponents we dont have
  if (!local)
    return false;
  else {
    // we report in any case, if the last remote report for an active sub component
    // was passive and we didn't report it as active before
    if (!global_active && !reported_active && local_active)
      return true;

    // if we were reporting last time - we do it again
    if (reported_by == _pid) {
      return true;
    }

    // otherwise see if no one else reported this sub component for some time
    timeval now = unixtime();
    timeval next_report = last_report;
    next_report.tv_sec += _interval * SUBCOMPONENT_REPORT_INTERVAL;
    #ifdef DEBUG4
    std::cout << "RtpiSubComponent(" << subid << "/" << alt_subid << ")::must_report: Now: " << now.tv_sec << " report at: " << next_report.tv_sec << std::endl;
    #endif
    return timercmp(&now,&next_report,>);
  }
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
bool RTPI::RtpiSubComponent::is_timed_out(int _interval) {
  #ifdef DEBUG2
  std::cout << "RtpiSubComponent(" << subid << "/" << alt_subid << ")::is_timed_out(" << _interval << ")" << std::endl;
  #endif

  // we do not remove these subcomponents
  if (local)
    return false;

  timeval now = unixtime();
  timeval t_out = timeout_base;
  t_out.tv_sec += _interval * SUBCOMPONENT_TIMEOUT_INTERVAL;

  #ifdef DEBUG4
  std::cout << "RtpiSubComponent(" << subid << "/" << alt_subid << ")::is_timed_out(): now: " << now.tv_sec << " timeout in: " << t_out.tv_sec << std::endl;
  #endif

  return timercmp(&now,&t_out,>);
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
RTPI::SubComponentMap::iterator RTPI::RtpiSubComponentSpace::find_component(uint32 _subid) {
  // seach our structure
  return subComponents.find(_subid);
};


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
RTPI::SubComponentMap::iterator RTPI::RtpiSubComponentSpace::altsubid_find_component(uint32 _alt_subid) {
  // seach our structure
  SubComponentIteratorMap::iterator spos = altsubid_subComponents.find(_alt_subid);
  if (spos != altsubid_subComponents.end()) {
    return spos->second;
  } else {
    return subComponents.end();
  }
};


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
RTPI::SubComponentMap::iterator RTPI::RtpiSubComponentSpace::insert_component(uint32 _subid, uint32 _alt_subid) {

  // create element
  std::pair<SubComponentMap::iterator, bool> res = subComponents.insert(SubComponentMap::value_type(_subid, RtpiSubComponent(_subid, _alt_subid)));

  if (res.second) {
    // creation was sucessfully
    if (!altsubid_subComponents.insert(SubComponentIteratorMap::value_type(_alt_subid, res.first)).second) {
      // should never happen
      #ifdef DEBUG1
      std::cout << "RtpiSubComponentSpace::insert_component(" << _subid << "): WARNING inconsistent subcomponent lists." << std::endl;
      #endif
      // remove existing entry
      altsubid_subComponents.erase(_alt_subid);
      // replace it with new one
      altsubid_subComponents.insert(SubComponentIteratorMap::value_type(_alt_subid, res.first));
    }
    return res.first;
  } else {
    return subComponents.end();
  }
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
RTPI::ParticipantSeqMapElement* RTPI::RtpiSubComponentSpace::find_participant_seq(uint32 _subid, u_int8 _type, u_int32 _pid) {

  // find subcomponent
  SubComponentMap::iterator spos = subComponents.find(_subid);
  if (spos == subComponents.end()) {
    // subcomponent does not exist (we don't create subcomponents here)!
    return NULL;
  }

  // find adu type
  ADUTypeSeqMap::iterator apos = spos->second.ADUtypes.find(_type);
  if (apos == spos->second.ADUtypes.end()) {
    // ADU type does not exist (we don't create ADU types here)!
    return NULL;
  }

  // find participant
  ParticipantSeqMap::iterator ppos = apos->second.participants.find(_pid);
  if (ppos == apos->second.participants.end()) {
    return NULL;
  } else {
    return &(*ppos);
  }
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
RTPI::ParticipantSeqMapElement* RTPI::RtpiSubComponentSpace::insert_participant_seq(uint32 _subid, u_int8 _type, u_int32 _pid) {

  // seach our structure
  // find subcomponent
  SubComponentMap::iterator spos = subComponents.find(_subid);
  if (spos == subComponents.end()) {
    // subcomponent does not exist (we don't create subcomponents here)!
    return NULL;
  }

  // find adu type
  ADUTypeSeqMap::iterator apos = spos->second.ADUtypes.find(_type);
  if (apos == spos->second.ADUtypes.end()) {
    // create element
    apos = spos->second.ADUtypes.insert(ADUTypeSeqMap::value_type(_type, RtpiADUType())).first;
  }
  // find participant
  ParticipantSeqMap::iterator ppos = apos->second.participants.find(_pid);
  if (ppos == apos->second.participants.end()) {
    // create element
    ppos = apos->second.participants.insert(ParticipantSeqMap::value_type(_type, RtpiParticipantSeq())).first;
  }

  return &(*ppos);
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void RTPI::RtpiSubComponentSpace::delete_participant_seq(u_int32 _pid) {
  // remove participant from all lists
  for (SubComponentMap::iterator scpos = subComponents.begin(); scpos != subComponents.end(); ++scpos) {
    for (ADUTypeSeqMap::iterator atpos = scpos->second.ADUtypes.begin(); atpos != scpos->second.ADUtypes.end(); ++atpos) {
      atpos->second.participants.erase(_pid);
    }
  }
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void RTPI::RtpiSubComponentSpace::delete_component(uint32 _subid) {
  // remove subcomponent
  altsubid_subComponents.erase(subComponents.find(_subid)->second.alt_subid);
  subComponents.erase(_subid);
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
RTPI::SubComponentMap::iterator RTPI::RtpiSubComponentSpace::delete_component(RTPI::SubComponentMap::iterator spos) {
  // remove subcomponent
  altsubid_subComponents.erase(spos->second.alt_subid);
  SubComponentMap::iterator tmp = spos;
  ++tmp;
  subComponents.erase(spos);
  return tmp;
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//* In combination with release_nonlocal_subcomps() these function
//* perform an release for all subcomponents.
//* However if an add_component is called after release_nonlocal_subcomps() the subcomponent
//* is restored in its previous state.
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void RTPI::RtpiSubComponentSpace::clear_subcomps_local() {
  // clear local flag for all subcomponents and remember local state
  for (SubComponentMap::iterator scpos = subComponents.begin(); scpos != subComponents.end(); ++scpos) {
    scpos->second.was_local = scpos->second.local;
    scpos->second.local = false;
  }
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//*
//*
//*
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void RTPI::RtpiSubComponentSpace::release_nonlocal_subcomps() {
  // release all nonlocal subcomponents having local_active set
  for (SubComponentMap::iterator scpos = subComponents.begin(); scpos != subComponents.end(); ++scpos) {
    if (!scpos->second.local && scpos->second.was_local) {
      scpos->second.release_component();
    }
    scpos->second.was_local = false;
  }
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void RTPI::RtpiSubComponentSpace::clear_subcomp_flag() {
  // clear flags
  for (SubComponentMap::iterator scpos = subComponents.begin(); scpos != subComponents.end(); ++scpos) {
    scpos->second.flag = false;
  }
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void RTPI::RtpiSubComponentSpace::release_all_components() {
  for (SubComponentMap::iterator scpos = subComponents.begin(); scpos != subComponents.end(); ++scpos) {
    scpos->second.release_component();
  }
}


