#ifdef WIN32
#include <winsock.h>
#include <io.h>
#else
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netinet/in.h>

//#ifndef INADRR_NONE
//#define INADDR_NONE 0xffffffff
//#endif

#endif

#include <util/iohelpers.h>
#include <rtpi/rtpiException.h>
#include <rtpi/rtpiUniqueId.h>

RTPI::UniqueIdClient::UniqueIdClient(std::string _address, int port) {
  // create address
  memset((char*)&address, 0, sizeof(address));	
  address.sin_family = AF_INET;
  address.sin_port = htons(port);
  address.sin_addr.s_addr = inet_addr(_address.c_str());

  if (address.sin_addr.s_addr == INADDR_NONE) {
    std::cerr << "UniqueIdClient::UniqueIdClient - given addresss is not a valid IP-Address" << std::endl;
    throw RtpiException(RtpiException::ERR_RTPI_SOCKET, "UniqueIdClient::UniqueIdClient - given addresss is not a valid IP-Address");
  }
}


bool RTPI::UniqueIdClient::Alive() {
  // init socket
  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    std::cerr << "UniqueIdClient::UniqueIdClient - error intializing socket" << std::endl;
    return false;
	  } 

  // connect to server
  if (connect(sock, (sockaddr*)&address, sizeof(address))<0) {
    std::cerr << "UniqueIdClient::UniqueIdClient - could not connect to server" << std::endl;
    #ifdef WIN32
    std::cerr << WSAGetLastError() << std::endl;
    #endif
    return false;
  }

  // close socket
  close(sock);
  
  return true;
}


std::vector<uint32>* RTPI::UniqueIdClient::GetIds(std::string scope, uint32 number) {
  // init socket
  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    std::cerr << "UniqueIdClient::UniqueIdClient - error intializing socket" << std::endl;
    throw RtpiException(RtpiException::ERR_RTPI_SOCKET, "UniqueIdClient::UniqueIdClient - error intializing socket");
  }

  // connect to server
  if (connect(sock, (sockaddr*)&address, sizeof(address))<0) {
    std::cerr << "UniqueIdClient::UniqueIdClient - could not connect to server" << std::endl;
    #ifdef WIN32
    std::cerr << WSAGetLastError() << std::endl;
    #endif
    return NULL;
  }

  // write request
  static char* sendbuf = new char[100];
  //static uint8*  s8  = (uint8*)sendbuf;
  static uint8 version = 0;
  static uint32* s32 = (uint32*)sendbuf;
  //s8[0]  = 0; // version
  s32[0] = htonl(number);
  s32[1] = htonl(scope.length());
  memcpy(sendbuf + 8, scope.c_str(), scope.length());

  if (writen(sock, &version, sizeof(version)) < sizeof(version)) {
    std::cerr << "UniqueIdClient::UniqueIdClient - error writing to server" << std::endl;
    #ifdef WIN32
    std::cerr << WSAGetLastError() << std::endl;
    #endif
    return NULL;
  }
  if (writen(sock, sendbuf, 8 + scope.length()) < 8 + scope .length()) {
    std::cerr << "UniqueIdClient::UniqueIdClient - error writing to server" << std::endl;
    #ifdef WIN32
    std::cerr << WSAGetLastError() << std::endl;
    #endif
    return NULL;
  }

  // read reply
  static char recvbuf[8]; // 2 * uint32
  static uint8 ok;
  static uint32 firstNumber;
  static uint32 size;
  if (readn(sock, &ok, sizeof(ok)) < sizeof(ok)) { // read reply byte
    std::cerr << "UniqueIdClient::UniqueIdClient - error reading from server" << std::endl;
    #ifdef WIN32
    std::cerr << WSAGetLastError() << std::endl;
    #endif
    return NULL;
  }

  if (!ok) {
    std::cerr << "UniqueIdClient::UniqueIdClient - number space full, no new numbers" << std::endl;
    return NULL;
  }

  if (readn(sock, &recvbuf, sizeof(recvbuf)) < sizeof(recvbuf)) { 
    std::cerr << "UniqueIdClient::UniqueIdClient - error reading from server" << std::endl;
    #ifdef WIN32
    std::cerr << WSAGetLastError() << std::endl;
    #endif
    return NULL;
  }

  firstNumber = ntohl(((uint32*)recvbuf)[0]);
  size = ntohl(((uint32*)recvbuf)[1]);
  
  std::vector<uint32>* numbers = new std::vector<uint32>;
  for (int i = 0; i < size; ++i) numbers->push_back(firstNumber + i);
  
  // close socket
  close(sock);
  
  return numbers;
}
