// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#include <rtpi/rtpiVirtual.h>
#include <rtpi/TclTimerManager.h>

//#define DEBUG_VR_M
//#define DEBUG_VR_S
//#define DEBUG_VR_AS
//#define DEBUG_VR_T

std::map<int, std::map<uint32, RTPI::_Subcomponent*> > RTPI::VirtualRtpi::subcomponentList; // subid, Subcomponent
std::map<int, std::map<uint32, RTPI::VirtualRtpi*> > RTPI::VirtualRtpi::participantList; // pid , VirtualRtpi
std::list<RTPI::RtcpiTimer*> RTPI::VirtualRtpi::timerList;

const uint32 RTPI::VirtualRtpi::minInt = 20;
const uint32 RTPI::VirtualRtpi::maxInt = 150;

std::map<int, bool> RTPI::VirtualRtpi::first;

RTPI::VirtualRtpi::VirtualRtpi() 
  : rtcpiReport(NULL), rtpiFeedback(NULL), rtcpiPacketFactory(NULL), rtpiPacketFactory(NULL), transport(NULL), joined(false), pid(0), transmissionTime(0) {
}


RTPI::VirtualRtpi::~VirtualRtpi() {
}


// group membership
uint32 RTPI::VirtualRtpi::join_group(int _sessionID, 
				     uint32 latency,
				     std::string username, 
				     RtcpiReportInterface* _rtcpiReport, 
				     RtpiFeedbackInterface* _rtpiFeedback, 
				     Transport* rtpiTransport) {

  if (joined) return 0;

  sessionID = _sessionID;

  bool found = false;
  std::map<uint32, VirtualRtpi*>& _participantList = participantList[sessionID];
  for (std::map<uint32, VirtualRtpi*>::iterator i = _participantList.begin(); i != _participantList.end(); ++i) {
    if (i->second == this) {
      found = true;
      break;
    }
  }
  
  if (!found) {
    pid = _rtcpiReport->get_new_pid();
    _participantList.insert(std::map<uint32, VirtualRtpi*>::value_type(pid, this));
  }

#ifdef DEBUG_VR_M
  FILE* debug = fopen("debugrtpi.txt", "a");
  fprintf(debug, "join_group - sessionID %u participant %u\n", sessionID, pid);
  fclose(debug);
#endif

  rtcpiReport = _rtcpiReport;
  rtpiFeedback = _rtpiFeedback;

  rtcpiPacketFactory = new RTPI::RtcpiPacketFactory();
  rtpiPacketFactory = new RTPI::RtpiPacketFactory();
  transport = rtpiTransport;
  transmissionTime = latency;
  
  joined = true;

  // report new member to group
  RtcpiTimer* timer;
  for (std::map<uint32, VirtualRtpi*>::iterator j = _participantList.begin(); j != _participantList.end(); ++j) {
    if ((j->second != this) && (j->second->joined)) {
      timer = new RtcpiTimer(RtcpiTimer::ADD_PARTICIPANT, pid, j->second->transmissionTime, sessionID, j->second->pid);
      timer->token = TclTimerManager::Tcl_CreateTimerHandler(timer->runningTime, ::RTPI::VirtualRtpi::ExecuteTimer, timer);
      timerList.push_back(timer);
#ifdef DEBUG_VR_T
      FILE* debug = fopen("debugrtpi.txt", "a");
      fprintf(debug, "join_group - set join timer: sessionID %u participant %u token %u\n", sessionID, j->second->pid, timer->token);
      fclose(debug);
#endif
    }
  }

  // set first sub report timer
  if (first.find(sessionID) == first.end()) {
    RtcpiTimer* timer = new RtcpiTimer(RtcpiTimer::REPORT, sessionID, 5000, sessionID, 0);
    timer->token = TclTimerManager::Tcl_CreateTimerHandler(timer->runningTime, ::RTPI::VirtualRtpi::ExecuteTimer, timer);
    timerList.push_back(timer);    
    first[sessionID] = false;
#ifdef DEBUG_VR_T
    FILE* debug = fopen("debugrtpi.txt", "a");
    fprintf(debug, "join_group - set subrep timer: sessionID %u token %u\n", sessionID, timer->token);
    fclose(debug);
#endif
  }

  return pid;
}


void RTPI::VirtualRtpi::leave_group() {
  if (!joined) return;

#ifdef DEBUG_VR_M
  FILE* debug = fopen("debugrtpi.txt", "a");
  fprintf(debug, "leave_group - sessionID %u participant %u\n", sessionID, pid);
  fclose(debug);
#endif

  delete transport; transport = NULL;
  delete rtpiPacketFactory; rtpiPacketFactory = NULL;
  delete rtcpiPacketFactory; rtcpiPacketFactory = NULL;
  joined = false;

  RtcpiTimer* timer;
  std::map<uint32, VirtualRtpi*>& _participantList = participantList[sessionID];
  for (std::map<uint32, VirtualRtpi*>::iterator j = _participantList.begin(); j != _participantList.end(); ++j) {
    if ((j->second != this) && (j->second->joined)) {
      timer = new RtcpiTimer(RtcpiTimer::REM_PARTICIPANT, pid, j->second->transmissionTime, sessionID, j->second->pid);
      timer->token = TclTimerManager::Tcl_CreateTimerHandler(timer->runningTime, ::RTPI::VirtualRtpi::ExecuteTimer, timer);
      timerList.push_back(timer);
#ifdef DEBUG_VR_T
      FILE* debug = fopen("debugrtpi.txt", "a");
      fprintf(debug, "leave_group - set leave timer: sessionID %u participant %u token %u\n", sessionID, j->second->pid, timer->token);
      fclose(debug);
#endif
    }
  }

  for (std::list<RtcpiTimer*>::iterator k = timerList.begin(); k != timerList.end(); ++k) {
    if (((*k)->sessionID == sessionID) && ((*k)->pid == pid)) {
#ifdef DEBUG_VR_T
      FILE* debug = fopen("debugrtpi.txt", "a");
      fprintf(debug, "leave_group - delete timer: token %u\n", (*k)->token);
      fclose(debug);
#endif
      TclTimerManager::Tcl_DeleteTimerHandler((*k)->token);
      delete *k; *k = NULL;
      k = timerList.erase(k);
    }
  }
  
  _participantList.erase(pid);
}


bool RTPI::VirtualRtpi::group_joined() {
  return (joined);
}


// manage participants
/*
RTPI::RtpiParticipant* RTPI::VirtualRtpi::get_own_participant() {
  if (!joined) return NULL;
  return (*rtpiHandler->getMasterParticipant());
}


RTPI::RtpiParticipant* RTPI::VirtualRtpi::get_participant(uint32 pid) {
  if (!joined) return NULL;
  RTPI::RtpiParticipantList::iterator participant = rtpiHandler->findParticipant(pid);
  return ((participant == rtpiHandler->getParticipantsEnd()) ? NULL : *participant);
}
*/

bool RTPI::VirtualRtpi::participant_exists(uint32 pid) {
  if (!joined) return false;

  std::map<uint32, VirtualRtpi*>& _participantList = participantList[sessionID];
  for (std::map<uint32, VirtualRtpi*>::iterator j = _participantList.begin(); j != _participantList.end(); ++j) {
    if ((j->second->joined) && (j->second->pid == pid)) return true;
  }  
  return false;
}


uint32 RTPI::VirtualRtpi::get_number_of_participants() {
  if (!joined) return 0;

  uint32 nbr = 0;
  std::map<uint32, VirtualRtpi*>& _participantList = participantList[sessionID];
  for (std::map<uint32, VirtualRtpi*>::iterator j = _participantList.begin(); j != _participantList.end(); ++j) {
    if (j->second->joined) ++nbr;
  }  

  return nbr;
}


/*
RTPI::RtpiParticipantList::iterator RTPI::VirtualRtpi::get_participants_begin() {
  if (!joined) return rtpiHandler->getParticipantsEnd();
  return rtpiHandler->getParticipantsBegin();
}


RTPI::RtpiParticipantList::iterator RTPI::VirtualRtpi::get_participants_end() {
  return rtpiHandler->getParticipantsEnd();
}


RTPI::RtpiParticipantList::iterator RTPI::VirtualRtpi::get_next_participant(RtpiParticipantList::iterator& i) {
  if (!joined) return rtpiHandler->getParticipantsEnd();
  return rtpiHandler->getNextParticipant(i);
}
*/

// manage subcomponents
void RTPI::VirtualRtpi::add_subcomponent(uint32 subid, std::string name, bool active) {
  if (!joined) return;

  std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[sessionID];
  std::map<uint32, _Subcomponent*>::iterator sub = _subcomponentList.find(subid);
  if (sub == _subcomponentList.end()) {
    _subcomponentList.insert(std::map<uint32, _Subcomponent*>::value_type(subid, new _Subcomponent(active, name)));
    sub = _subcomponentList.find(subid);
  } else {
    sub->second->active = active;
    if (name.length() != 0) sub->second->name = name;
    return;
  }

#ifdef DEBUG_VR_S
  FILE* debug = fopen("debugrtpi.txt", "a");
  fprintf(debug, "add_subcomponent - sessionID %u participant %u subid %u\n", sessionID, pid, subid);
  fclose(debug);
#endif

  RtcpiTimer* timer;
  std::map<uint32, VirtualRtpi*>& _participantList = participantList[sessionID];
  for (std::map<uint32, VirtualRtpi*>::iterator j = _participantList.begin(); j != _participantList.end(); ++j) {
    if ((j->second != this) && (j->second->joined)) {
      timer = new RtcpiTimer(RtcpiTimer::ADD_SUBCOMPONENT, subid, j->second->transmissionTime, sessionID, j->second->pid);
      timer->token = TclTimerManager::Tcl_CreateTimerHandler(timer->runningTime, ::RTPI::VirtualRtpi::ExecuteTimer, timer);
      timerList.push_back(timer);
#ifdef DEBUG_VR_T
      FILE* debug = fopen("debugrtpi.txt", "a");
      fprintf(debug, "add_subcomponent - set add_sub timer: sessionID %u participant %u token %u\n", sessionID, j->second->pid, timer->token);
      fclose(debug);
#endif
    }
  }
}


void RTPI::VirtualRtpi::delete_subcomponent(uint32 subid) {
  if (!joined) return;

  std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[sessionID];
  std::map<uint32, _Subcomponent*>::iterator sub = _subcomponentList.find(subid);
  if (sub != _subcomponentList.end()) {
    delete sub->second;
    _subcomponentList.erase(sub);
  } else {
    //return;
  }

  RtcpiTimer* timer;
  std::map<uint32, VirtualRtpi*>& _participantList = participantList[sessionID];
  for (std::map<uint32, VirtualRtpi*>::iterator j = _participantList.begin(); j != _participantList.end(); ++j) {
    if ((j->second != this) && (j->second->joined)) {
      timer = new RtcpiTimer(RtcpiTimer::REM_SUBCOMPONENT, subid, j->second->transmissionTime, sessionID, j->second->pid);
      timer->token = TclTimerManager::Tcl_CreateTimerHandler(timer->runningTime, ::RTPI::VirtualRtpi::ExecuteTimer, timer);
      timerList.push_back(timer);
#ifdef DEBUG_VR_T
      FILE* debug = fopen("debugrtpi.txt", "a");
      fprintf(debug, "delete_subcomponent - set rem_sub timer: sessionID %u participant %u token %u\n", sessionID, j->second->pid, timer->token);
      fclose(debug);
#endif
    }
  }
}

/*
RTPI::RtpiSubComponent* RTPI::VirtualRtpi::get_subcomponent(uint32 subid) {
  if (!joined) return NULL;
  SubComponentMap::iterator sub = rtpiHandler->get_seq_space()->find_component(subid);
  return ((sub == rtpiHandler->get_seq_space()->get_components_end()) ? NULL : &(sub->second));
}
*/

bool RTPI::VirtualRtpi::subcomponent_exists(uint32 subid) {
  if (!joined) return false;

  std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[sessionID];
  if (_subcomponentList.find(subid) != _subcomponentList.end()) return true;
  return false;
}


uint32 RTPI::VirtualRtpi::get_number_of_subcomponents() {
  if (!joined) return 0;

  std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[sessionID];
  return _subcomponentList.size();
}

/*    
RTPI::SubComponentMap::iterator RTPI::VirtualRtpi::get_subcomponents_begin() {
  if (!joined) return rtpiHandler->get_seq_space()->get_components_end();
  return rtpiHandler->get_seq_space()->get_components_begin();
}


RTPI::SubComponentMap::iterator RTPI::VirtualRtpi::get_subcomponents_end() {
  return rtpiHandler->get_seq_space()->get_components_end();
}


RTPI::SubComponentMap::iterator RTPI::VirtualRtpi::get_next_subcomponent(SubComponentMap::iterator& i) {
  if (!joined) return rtpiHandler->get_seq_space()->get_components_end();
  return ++i;
}
*/

void RTPI::VirtualRtpi::activate_subcomponent(uint32 subid) {
  if (!joined) return;

  std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[sessionID];
  std::map<uint32, _Subcomponent*>::iterator sub = _subcomponentList.find(subid);
  if (sub == _subcomponentList.end()) {
    _subcomponentList.insert(std::map<uint32, _Subcomponent*>::value_type(subid, new _Subcomponent(true, "")));
    sub = _subcomponentList.find(subid);
  } else {
    if (sub->second->active) return; //already active
    sub->second->active = true;
  }

  RtcpiTimer* timer;
  std::map<uint32, VirtualRtpi*>& _participantList = participantList[sessionID];
  for (std::map<uint32, VirtualRtpi*>::iterator j = _participantList.begin(); j != _participantList.end(); ++j) {
    if ((j->second != this) && (j->second->joined)) {
      timer = new RtcpiTimer(RtcpiTimer::ACT_SUBCOMPONENT, subid, j->second->transmissionTime, sessionID, j->second->pid);
      timer->token = TclTimerManager::Tcl_CreateTimerHandler(timer->runningTime, ::RTPI::VirtualRtpi::ExecuteTimer, timer);
      timerList.push_back(timer);
#ifdef DEBUG_VR_T
      FILE* debug = fopen("debugrtpi.txt", "a");
      fprintf(debug, "activate_subcomponent - set act_sub timer: sessionID %u participant %u token %u\n", sessionID, j->second->pid, timer->token);
      fclose(debug);
#endif
    }
  }
}


void RTPI::VirtualRtpi::deactivate_subcomponent(uint32 subid) {
  if (!joined) return;

  std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[sessionID];
  std::map<uint32, _Subcomponent*>::iterator sub = _subcomponentList.find(subid);
  if (sub == _subcomponentList.end()) {
    _subcomponentList.insert(std::map<uint32, _Subcomponent*>::value_type(subid, new _Subcomponent(false, "")));
    sub = _subcomponentList.find(subid);
  } else {
    if (!sub->second->active) return; //already passive
    sub->second->active = false;
  }

  RtcpiTimer* timer;
  std::map<uint32, VirtualRtpi*>& _participantList = participantList[sessionID];
  for (std::map<uint32, VirtualRtpi*>::iterator j = _participantList.begin(); j != _participantList.end(); ++j) {
    if ((j->second != this) && (j->second->joined)) {
      timer = new RtcpiTimer(RtcpiTimer::DEA_SUBCOMPONENT, subid, j->second->transmissionTime, sessionID, j->second->pid);
      timer->token = TclTimerManager::Tcl_CreateTimerHandler(timer->runningTime, ::RTPI::VirtualRtpi::ExecuteTimer, timer);
      timerList.push_back(timer);
#ifdef DEBUG_VR_T
      FILE* debug = fopen("debugrtpi.txt", "a");
      fprintf(debug, "deactivate_subcomponent - set dea_sub timer: sessionID %u participant %u token %u\n", sessionID, j->second->pid, timer->token);
      fclose(debug);
#endif
    }
  }
}


bool RTPI::VirtualRtpi::subcomponent_is_active(uint32 subid) {
  if (!joined) return false;

  std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[sessionID];
  std::map<uint32, _Subcomponent*>::iterator sub = _subcomponentList.find(subid);
  if (sub == _subcomponentList.end()) return false;
  return sub->second->active;
}


void RTPI::VirtualRtpi::set_subcomponent_name(uint32 subid, std::string name) {
  if (!joined) return;

  std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[sessionID];
  std::map<uint32, _Subcomponent*>::iterator sub = _subcomponentList.find(subid);
  if (sub == _subcomponentList.end()) return;

  sub->second->name = name;
}


std::string RTPI::VirtualRtpi::get_subcomponent_name(uint32 subid) {
  if (!joined) return "";

  std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[sessionID];
  std::map<uint32, _Subcomponent*>::iterator sub = _subcomponentList.find(subid);
  if (sub == _subcomponentList.end()) return "";
  return sub->second->name;
}


bool RTPI::VirtualRtpi::subcomponent_has_name(uint32 subid) {
  if (!joined) return false;

  std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[sessionID];
  std::map<uint32, _Subcomponent*>::iterator sub = _subcomponentList.find(subid);
  if (sub == _subcomponentList.end()) return false;
  return ((sub->second->name.length() == 0) ? false : true);
}


// helper functions
void RTPI::VirtualRtpi::send_rtcpi_note(std::string note) {
  if (!joined) return;
}


uint32 RTPI::VirtualRtpi::get_current_time() {
  return rtpitime();
}


// outgoing data
RefCount<RTPI::BufferVector> RTPI::VirtualRtpi::create_rtpi_packets(RefCount<RtpiAdu> adu) {
  if (!joined || !adu) return NULL;
  
  RefCount<RtpiPacketVector> rtpiPackets = transport->CreateRtpiPackets(adu);

  RefCount<BufferVector> bufVector = new BufferVector();
  for (RtpiPacketVector::iterator i = rtpiPackets->begin(); i != rtpiPackets->end(); ++i)
    bufVector->push_back((*i)->getBuffer());

  return bufVector;
}


RefCount<RTPI::BufferVector> RTPI::VirtualRtpi::create_rtpi_packets(RefCount<RtpiAduVector> adus) {
  if (!joined || !adus || (adus->size() == 0)) return NULL;
  
  RefCount<RtpiCompoundPacketVector> rtpiPackets = transport->CreateRtpiPackets(adus);
  
  RefCount<BufferVector> bufVector = new BufferVector();
  for (RtpiCompoundPacketVector::iterator i = rtpiPackets->begin(); i != rtpiPackets->end(); ++i)
    bufVector->push_back((*i)->GetBuffer());
  
  return bufVector;
}


// incoming data
RefCount<RTPI::RtpiAduVector> RTPI::VirtualRtpi::create_rtpi_adus(RefCount<Buffer> buf) {
  if (!joined || !buf) return NULL;
  return (transport->CreateRtpiAdusFromPackets(rtpiPacketFactory->CreateRtpiPackets(buf)));
}


void RTPI::VirtualRtpi::handle_received_rtpi(RefCount<RtpiAduVector> adus, uint32 senderIpAddress) {
  if (!joined || !adus) return;
}


void RTPI::VirtualRtpi::handle_received_rtcpi(RefCount<Buffer> buf, uint32 senderIpAddress) {
  if (!joined || !buf) return;
}



void RTPI::VirtualRtpi::ExecuteTimer(ClientData data) {
  RtcpiTimer* expiredTimer = (RtcpiTimer*)data;

  // delete expired timer from timerList
  for (std::list<RtcpiTimer*>::iterator i = timerList.begin(); i != timerList.end(); ++i) {
    if ((*i)->id == expiredTimer->id) {
      timerList.erase(i);
      break;
    }  
  }

#ifdef DEBUG_VR_T
  FILE* debug = fopen("debugrtpi.txt", "a");
  fprintf(debug, "ExectuteTimer - exec timer: token %u\n", expiredTimer->token);
  fclose(debug);
#endif

  switch (expiredTimer->type) {
  case RtcpiTimer::ADD_PARTICIPANT: {
    std::map<uint32, VirtualRtpi*>& _participantList = participantList[expiredTimer->sessionID];
    std::map<uint32, VirtualRtpi*>::iterator _participant = _participantList.find(expiredTimer->pid);
    if ((_participant == _participantList.end()) || !_participant->second->joined) break;
    _participant->second->rtpiFeedback->participant_added(expiredTimer->psid);
    break;
  }
  case RtcpiTimer::REM_PARTICIPANT: {
    std::map<uint32, VirtualRtpi*>& _participantList = participantList[expiredTimer->sessionID];
    std::map<uint32, VirtualRtpi*>::iterator _participant = _participantList.find(expiredTimer->pid);
    if ((_participant == _participantList.end()) || !_participant->second->joined) break;
    _participant->second->rtpiFeedback->participant_removed(expiredTimer->psid);
    break;
  }
  case RtcpiTimer::ADD_SUBCOMPONENT: {
    std::map<uint32, VirtualRtpi*>& _participantList = participantList[expiredTimer->sessionID];
    std::map<uint32, VirtualRtpi*>::iterator _participant = _participantList.find(expiredTimer->pid);
    if ((_participant == _participantList.end()) || !_participant->second->joined) break;
#ifdef DEBUG_VR_S
    FILE* debug = fopen("debugrtpi.txt", "a");
    fprintf(debug, "ExecuteTimer - add_subcomponent - sessionID %u participant %u subid %u\n", 
	    _participant->second->sessionID, _participant->second->pid, expiredTimer->psid);
    fclose(debug);
#endif
    std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[_participant->second->sessionID];
    std::map<uint32, _Subcomponent*>::iterator isub = _subcomponentList.find(expiredTimer->psid);
    if (isub == _subcomponentList.end()) {
      break;
    }
    _Subcomponent* sub = isub->second;
    RtpiSubComponent rtpiSub(expiredTimer->psid);
    rtpiSub.add_component(sub->active, (char*)sub->name.c_str(), sub->name.length());
    std::pair<const uint32, RtpiSubComponent>* element = new std::pair<const uint32, RtpiSubComponent>(expiredTimer->psid, rtpiSub);
    _participant->second->rtpiFeedback->subcomponent_added(element, RTPI::RtpiFeedbackInterface::PKT_RTCPI);
    delete element;
    break;
  }
  case RtcpiTimer::REM_SUBCOMPONENT: {
    std::map<uint32, VirtualRtpi*>& _participantList = participantList[expiredTimer->sessionID];
    std::map<uint32, VirtualRtpi*>::iterator _participant = _participantList.find(expiredTimer->pid);
    if ((_participant == _participantList.end()) || !_participant->second->joined) break;
    _participant->second->rtpiFeedback->subcomponent_removed(expiredTimer->psid);
    break;
  }
  case RtcpiTimer::ACT_SUBCOMPONENT: {
    std::map<uint32, VirtualRtpi*>& _participantList = participantList[expiredTimer->sessionID];
    std::map<uint32, VirtualRtpi*>::iterator _participant = _participantList.find(expiredTimer->pid);
    if ((_participant == _participantList.end()) || !_participant->second->joined) break;
#ifdef DEBUG_VR_AS
    FILE* debug = fopen("debugrtpi.txt", "a");
    fprintf(debug, "ExecuteTimer - act_subcomponent - sessionID %u participant %u subid %u\n", 
	    _participant->second->sessionID, _participant->second->pid, expiredTimer->psid);
    fclose(debug);
#endif
    std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[_participant->second->sessionID];
    std::map<uint32, _Subcomponent*>::iterator isub = _subcomponentList.find(expiredTimer->psid);
    if (isub == _subcomponentList.end()) {
      break;
    }
    _Subcomponent* sub = isub->second;
    RtpiSubComponent rtpiSub(expiredTimer->psid);
    rtpiSub.add_component(true, (char*)sub->name.c_str(), sub->name.length());
    std::pair<const uint32, RtpiSubComponent>* element = new std::pair<const uint32, RtpiSubComponent>(expiredTimer->psid, rtpiSub);
    _participant->second->rtpiFeedback->subcomponent_changed(element, RTPI::RtpiFeedbackInterface::PKT_RTCPI);
    delete element;
    break;
  }
  case RtcpiTimer::DEA_SUBCOMPONENT: {
    std::map<uint32, VirtualRtpi*>& _participantList = participantList[expiredTimer->sessionID];
    std::map<uint32, VirtualRtpi*>::iterator _participant = _participantList.find(expiredTimer->pid);
    if ((_participant == _participantList.end()) || !_participant->second->joined) break;
#ifdef DEBUG_VR_AS
    FILE* debug = fopen("debugrtpi.txt", "a");
    fprintf(debug, "ExecuteTimer - dea_subcomponent - sessionID %u participant %u subid %u\n", 
	    _participant->second->sessionID, _participant->second->pid, expiredTimer->psid);
    fclose(debug);
#endif
    std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[_participant->second->sessionID];
    std::map<uint32, _Subcomponent*>::iterator isub = _subcomponentList.find(expiredTimer->psid);
    if (isub == _subcomponentList.end()) {
      break;
    }
    _Subcomponent* sub = isub->second;
    RtpiSubComponent rtpiSub(expiredTimer->psid);
    rtpiSub.add_component(false, (char*)sub->name.c_str(), sub->name.length());
    std::pair<const uint32, RtpiSubComponent>* element = new std::pair<const uint32, RtpiSubComponent>(expiredTimer->psid, rtpiSub);
    _participant->second->rtpiFeedback->subcomponent_changed(element, RTPI::RtpiFeedbackInterface::PKT_RTCPI);
    delete element;
    break;
  }
  case RtcpiTimer::REPORT: {
    std::map<uint32, VirtualRtpi*>& _participantList = participantList[expiredTimer->psid];
    std::map<uint32, _Subcomponent*>& _subcomponentList = subcomponentList[expiredTimer->psid];
    _Subcomponent* sub;
    for (std::map<uint32, _Subcomponent*>::iterator i = _subcomponentList.begin(); i != _subcomponentList.end(); ++i) {
      sub = i->second;
      RtpiSubComponent rtpiSub(i->first);
      rtpiSub.add_component(sub->active, (char*)sub->name.c_str(), sub->name.length());
      std::pair<const uint32, RtpiSubComponent>* element = new std::pair<const uint32, RtpiSubComponent>(i->first, rtpiSub);
      for (std::map<uint32, VirtualRtpi*>::iterator j = _participantList.begin(); j != _participantList.end(); ++j) {
	if (!j->second->joined) continue;
	j->second->rtpiFeedback->subcomponent_added(element, RTPI::RtpiFeedbackInterface::PKT_RTCPI);
      }
      delete element;
    }

    // next round
    RtcpiTimer* timer = new RtcpiTimer(RtcpiTimer::REPORT, expiredTimer->psid, 5000, expiredTimer->psid, 0);
    timer->token = TclTimerManager::Tcl_CreateTimerHandler(timer->runningTime, ::RTPI::VirtualRtpi::ExecuteTimer, timer);
    timerList.push_back(timer);    
#ifdef DEBUG_VR_T
    FILE* debug = fopen("debugrtpi.txt", "a");
    fprintf(debug, "ExecuteTimer - set subrep timer: sessionID %u token %u\n", expiredTimer->psid, timer->token);
    fclose(debug);
#endif
  }
  }


  delete expiredTimer; expiredTimer = NULL;
}
