// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Christoph Kuhmuench, Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {cjk,hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#ifndef _REF_COUNT_CC_
#define _REF_COUNT_CC_

//#define DEBUG_MODE

template<class T>
class RefCount
{

  class RefCountBody
  {
  public:
    int  counter;
    T   *t;

  public:
    RefCountBody(T* _t) : counter(0), t(_t)
      {};

    ~RefCountBody()
      { if (t != NULL) delete t; };
  };

private:
  RefCountBody *rcb;

public:
  RefCount() : rcb( NULL )
    { };

  RefCount(T* _t) : rcb( ((_t != NULL) ? new RefCountBody(_t) : NULL) )
    { if (rcb != NULL) rcb->counter++; };

  RefCount(const RefCount<T>& rc) : rcb(rc.rcb)
    { if (rcb != NULL) rcb->counter++; };

  RefCount<T>& operator=(const RefCount<T>& rc)
    {
      if (this != &rc) {
        if (rcb != NULL && --rcb->counter <= 0) delete rcb;
        rcb = rc.rcb;
        if (rcb != NULL) rcb->counter++;
      }
      return *this;
    };

  RefCount<T>& operator=(T* _t)
    {
      if (rcb != NULL && --rcb->counter <= 0) delete rcb;

      if (_t != NULL) {
	rcb = new RefCountBody(_t);
	rcb->counter++;
      } else {
	rcb = NULL;
      }
      return *this;
    };

  bool operator==(const RefCount<T>& rc) const {
    return (rcb == rc.rcb);
  };

  bool operator!=(const RefCount<T>& rc) const {
    return (rcb != rc.rcb);
  };

  bool operator!() const {
    return (rcb == NULL);
  }

  ~RefCount()
    { if (rcb != NULL && --rcb->counter <= 0) delete rcb; };


  T& operator*() {
    #ifdef DEBUG_MODE
    if ((rcb == NULL) || (rcb->t == NULL)) {
      //cout << "NULL pointer exception in RefCount::operator*()!" << endl;
      exit(1);
    }
    #endif
    return *rcb->t;
  };

  T* operator->() {
    #ifdef DEBUG_MODE
    if ((rcb == NULL) || (rcb->t == NULL)) {
      //cout << "NULL pointer exception in RefCount::operator->()!" << endl;
      exit(1);
    }
    #endif
    return rcb->t;
  };
};

#endif
