// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#include "Buffer.h" 

#include <iostream>

#ifndef WIN32
#include <string.h>
#endif

#define xDEBUG0
#define xDEBUG1
#define xDEBUG2
#define xDEBUG3
#define xDEBUG4


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
Buffer::Buffer()
{
  #ifdef DEBUG2
  cout << "Buffer()::Buffer()" << endl;
  #endif

  bufferSize = DEFAULT_BUFFER_SIZE;
  size = bufferSize;
  if (bufferSize==0) {
    buf = NULL;
    begin = NULL;
  } else {
    buf = new char[bufferSize];
    begin = buf;
  };

  #ifdef DEBUG4
  cout << "Buffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
  #endif
};

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
Buffer::Buffer(int bs)  	// initial media buffer size
{
  #ifdef DEBUG2
  cout << "Buffer()::Buffer(" << bs << ")" << endl;
  #endif

  bufferSize = bs;
  size = bufferSize;
  if (bufferSize==0) {
    buf = NULL;
    begin = NULL;
  } else {
    buf = new char[bufferSize];
    begin = buf;
  };

  #ifdef DEBUG4
  cout << "Buffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
  #endif
};

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
Buffer::Buffer(char* b, int bs)
{
  #ifdef DEBUG2
  cout << "Buffer()::Buffer(" << b << ", " << bs << ")" << endl;
  #endif

  bufferSize = bs;
  size = bufferSize;
  if (bufferSize == 0) {
    buf = NULL;
    begin = NULL;
  } else {
    buf = new char[bufferSize];
    begin = buf;
    memcpy(buf, b, bufferSize);
  };

  #ifdef DEBUG4
  cout << "Buffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
  #endif
};

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
Buffer::Buffer(Buffer& b)  	// initial media buffer bufferSize
{
  #ifdef DEBUG2
  cout << "Buffer()::Buffer(Buffer&)" << endl;
  #endif

  bufferSize = b.bufferSize;
  size = b.size;
  if (bufferSize == 0) {
    buf = NULL;
    begin = NULL;
  } else {
    buf = new char[bufferSize];
    begin = b.begin;
    memcpy(buf, b.buf, bufferSize);
  };

  #ifdef DEBUG4
  cout << "Buffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
  #endif
};

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
Buffer::~Buffer() {
  #ifdef DEBUG2
  cout << "Buffer::~Buffer() (size was: " << bufferSize << ")" << endl;
  #endif
  #ifdef DEBUG4
  cout << "Buffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
  #endif

  clear();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
Buffer* Buffer::subArea(int start, int length)
{
  #ifdef DEBUG2
  cout << "Buffer::subArea()" << endl;
  #endif

  Buffer *b = new Buffer(length);
  if (b->bufferSize != 0)
    memcpy(b->buf, begin + start, b->bufferSize);
  return b;
}

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void Buffer::clear() {
  #ifdef DEBUG2
  cout << "Buffer::clear()" << endl;
  #endif
  #ifdef DEBUG4
  cout << "Buffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
  #endif

  bufferSize = 0;
  size = 0;
  if (buf)
    delete[] buf;
  buf = NULL;
  begin = NULL;
};

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
bool Buffer::resize(int new_size) {
  #ifdef DEBUG3
  cout << "Buffer::resize("<< new_size <<")" << endl;
  #endif
  #ifdef DEBUG4
  cout << "oldBuffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
  #endif

  int offset = begin - buf; // dead space at beginning of buf -> (bufferSize-offset) is the actual usable memory space 

  if (bufferSize - offset < new_size) {
    char* oldBuf = buf;
    buf = new char[new_size];
    memcpy(buf, begin, size);
    delete[] oldBuf;
    begin = buf;
    // keep size if it was modified
    size = ((bufferSize == size) ? new_size : size);
    bufferSize = new_size;

    #ifdef DEBUG4
    cout << "resizedBuffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
    #endif

    return true;
  } else {
    // we just update size
    size = new_size;
  }

  #ifdef DEBUG4
  cout << "resizedBuffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
  #endif

  return false;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void Buffer::shiftCont(int offset) {
  #ifdef DEBUG3
  cout << "Buffer::shiftCont("<< offset <<")" << endl;
  #endif
  #ifdef DEBUG4
  cout << "oldBuffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
  #endif

  if (offset == 0) // nothing to do
    return;
  if ((bufferSize == 0) && (offset < 0))  // this is an error!
    return;
  if ((bufferSize == 0) && (offset > 0)) {  // is an error but ... ok
    // no buffer was allocated -> allocate one
    bufferSize = offset;
    size = bufferSize;
    buf = new char[bufferSize];
    begin = buf;
    return;
  }

  int existingOffset = begin - buf; // dead space at beginning of buf

  // positive offset -> move backwards
  if (offset > 0) {

    if (existingOffset >= offset) {
      // there is enough dead space at beginning of buffer -> just reposition begin-pointer
      begin -= offset;
      size += offset;

      #ifdef DEBUG4
      cout << "resizedBuffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
      #endif
      return;
    }

    // dead space at beginning is not big enough -> real memmove operations necessary

    if (bufferSize - size >= offset) {
      // combined dead space at beginning and end of the buffer is sufficient
      int remainingOffset = offset - existingOffset;

      memmove(begin + remainingOffset, begin, size);
      begin = buf;  // data starts now at beginning of buffer
      size += offset;

      #ifdef DEBUG4
      cout << "resizedBuffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
      #endif
      return;
    }

    // combined dead space at beginning and end of the buffer is not sufficient -> create new buffer
    char* old_buf = buf;
    bufferSize = size + offset;
    buf = new char[bufferSize];
    memmove(buf + offset, begin, size);      //  move backwards
    begin = buf;
    size += offset;
    delete[] old_buf;

    #ifdef DEBUG4
    cout << "resizedBuffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
    #endif
    return;
  }

  // negative offset -> cut off at the beginning by repositioning the begin-pointer
  begin -= offset; // offset is < 0;
  size += offset;

  /* real movement operation
  memmove(buf, begin - offset, size + offset); // offset is < 0!
  begin = buf; 
  size += offset;
  */

  #ifdef DEBUG4
  cout << "resizedBuffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
  #endif

  return;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void Buffer::setCont(char* b, int bs)
{
  #ifdef DEBUG2
  cout << "Buffer()::setCont(" << b << ", " << bs << ")" << endl;
  #endif
  #ifdef DEBUG4
  cout << "oldBuffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
  #endif

  // erase previous content
  clear();

  bufferSize = bs;
  size = bufferSize;
  if (bufferSize == 0) {
    buf = NULL;
    begin = NULL;
  } else {
    buf = new char[bufferSize];
    begin = buf;
    memcpy(buf, b, bufferSize);
  };

  #ifdef DEBUG4
  cout << "resizedBuffer: OverallSize " << bufferSize << " Size " << size << " Memory " << (void*)buf << " Begin of Data " << (void*)begin << endl;
  #endif
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void Buffer::setBegin(char* b) { 
  // set beginning of data in buffer
  if ((b > buf + bufferSize) || (b < buf))
    return; // error

  begin = b;
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void Buffer::setBegin(int offset) { 
  // set beginning of data in buffer
  if ((begin + offset > buf + bufferSize) || (begin + offset < buf))
    return; // error

  begin += offset;
}


