// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#ifndef _Buffer_h_
#define _Buffer_h_

#include <stdio.h>
#include <stdlib.h>

#ifdef WIN32
#include <memory.h>
#include <string.h>
#endif

#ifdef _SunOS_IRIX_
#include <strings.h> // for memmove
#endif

#ifdef _Linux_
#include <string.h> // for memmove
#endif

#define DEFAULT_BUFFER_SIZE 8192

//#ifdef WIN32
//#include <util/refcount.cpp>
//#else
//#include <util/refcount.cc>
//#endif


#include <vector>

class Buffer {
  public:
    Buffer();
    Buffer(int);	// set default buffer size
    Buffer(char*, int);
    Buffer(Buffer&);  	// copy constructor
    ~Buffer();

    int getSize() const { return size; }
    void setSize(int s) { size = s; }    // set size to length of data
    char* getCont() { return begin; }
    void setCont(char*, int);
    void setBegin(char*);
    void setBegin(int);

    Buffer* subArea(int, int);

    void clear();
    bool isEmpty() const { return (buf == NULL); }
    void reset() { size = bufferSize; }  // here we simply reset size
    bool resize(int);
    void shiftCont(int);

  private:
    int         bufferSize;  // size of allocated memory
    int         size;        // may be changed to indicate length of data
    char*       buf;         // pointer to begin of Buffer
    char*       begin;       // pointer to begin of data (might be greater than begin of Buffer)
};


//typedef std::vector<RefCount<Buffer> > BufferVector;

#endif




