
/**
 ****************************************************************************
 *                                                                          *
 *  template class list                                                     *
 *  -------------------                                                     *
 *                                                                          *
 ****************************************************************************
 */

#ifndef _LIST_C_
#define _LIST_C_

#include<iostream>

#ifndef _UINT_
#define _UINT_
typedef unsigned int uint;
#endif

// typedef int T;

/*
#ifndef _ERROREXIT_
#define _ERROREXIT_
int errorexit(char * s)
{
  cerr << s << endl;
  exit(1);
}
#endif
*/

/// class listiterator; weiter unten

template<class T>
class listiterator;


/// Doppelt verkettete Liste
template<class T>
class list
{
#ifdef _LINUX_
    friend ostream& operator <<(ostream& os, list& l);
    friend istream& operator >>(istream& is, list& l);
#endif
#ifdef _SunOS_IRIX_
    friend ostream& operator <<<>(ostream& os, list& l);
    friend istream& operator >><>(istream& is, list& l);
#endif

    friend class listiterator<T>;
  /// symbolisiert ein Listenelement
    struct ele
    {   T    t;     /// Element
        ele *next;  /// Vorwrtszeiger
        ele *prev;  /// ckwrtszeiger

        ele() : next(NULL), prev(NULL) {;};
        ele(const T& _t, ele *_n, ele *_p) : t(_t), next(_n), prev(_p) {;};
    };

    private:
        ele  *head;  /// Anfang der Liste
        ele  *tail;  /// Ende der Liste
        uint n;      /// Anzahl der Elemente

    public:
///      list(void);   Der Sche... g++ verstehts nicht anders!
        list(void) : head(new ele), tail(new ele), n(0)
	  {
	    head->next= tail->next= tail;
	    head->prev= tail->prev= head;
	  };

        list(const list& l);
        ~list();

        inline int size()  { return getLen(); };
  
        int    getLen()    { return n; };

        list& operator=(const list& l);

        int operator==(const list& l);
        int operator!=(const list& l);
        
        list& operator+=(const list& l);
        list& operator+=(const T& t) { insert(t); return *this; };
        list& operator-=(const T& t) { remove(t); return *this; };
        list& operator+ (const list& l);

        T&    operator[](unsigned long i);  

        int  addElement(const T& t)    { return insert(t); };
        int  removeElement(const T& t) { return remove(t); };  
        int  push_back(const T& t);
        int  insert(const T& t);
        int  remove(const T& t);
        int  search(const T& t);
        int  retrieve(T& t);
        int  retrieve(T** t);
        void eraselist();
};

/// TemplateKlasse zum Durchlaufen von Listen
template<class T>
class listiterator
{
private:
  list<T>      *l;   /// Pointer auf Liste
  list<T>::ele *akt; /// aktuelles Element in der Liste
  int           pos; /// Positionsnummer

public:
  listiterator()            : l(NULL), akt(NULL),    pos(0) {};
  listiterator(list<T> *_l) : l(_l),   akt(l->head), pos(0) {};

  int gotostart();
  int hasMoreElements() { return !EOL();        };
  T&  nextElement()     { return operator++(1); };
  
  
  int EOL() { return akt == l->tail; };  /// End Of List?
  T&  operator++(int);
  T&  operator--(int);
  T&  get() { return akt->t; };

};

template<class T>
int listiterator<T>::gotostart()
{
  akt= l->head->next;
  pos= 1;

  return akt != l->tail;
};

template<class T>
T& listiterator<T>::operator++(int)
{
  T& t= akt->t;
  akt= akt->next;
  return t;
};
    
template<class T>
T& listiterator<T>::operator--(int)
{
  T& t= akt->t;
  akt= akt->prev;
  return t;
};



/**
 **********************************************************************
 * friends: io-operationen                                            *
 **********************************************************************
 */

template<class T>
ostream& operator <<(ostream& os, list<T>& l)
{
    os << "[ " << l.n << " ]{ ";
    
    list<T>::ele *tmp= l.head->next;
    if (tmp != l.tail)
      { os << tmp->t;
        for (tmp= tmp->next; tmp != l.tail; tmp= tmp->next)
            os << "   " << tmp->t;
      }

    os << " }" << std::endl;

    return os;
};

template<class T>
istream& operator >>(istream& is, list<T>& l)
{
    char buf[256];
    uint num;
    T    tb;
    
    is >> buf;
    if (buf[0] != '[')
        errorexit("Syntaxfehler beim Einlesen.(1)");

    is >> num;
    is >> buf;
    if ((buf[0] != ']') && (buf[1] != '{'))
        errorexit("Syntaxfehler beim Einlesen.(2)");

    /// Alten Inhalt loeschen:
    l.eraselist();

    /// Jetzt Einlesen:
    for (int i= 0; i < num ; i++)
      { is >> tb;
        l.insert(tb);
      }
    
    is >> buf;
    if (buf[0] != '}')
        errorexit("Syntaxfehler beim Einlesen.(3)");

    return is;
};


/**
 **********************************************************************
 * Konstruktoren, Destruktor                                          *
 **********************************************************************
 */

// template<class T>
// list<T>::list(void) : head(new ele), tail(new ele), n(0)
// {
//     head->next= tail->next= tail;
//     head->prev= tail->prev= head;
// };
/// Konstruktor
template<class T>
list<T>::list(const list& l) : head(new ele), tail(new ele), n(l.n)
{
    head->next= tail->next= tail;
    head->prev= tail->prev= head;

    ele *tmp1= head;
    for (ele *tmp2= l.head->next; tmp2 != l.tail; tmp2= tmp2->next)
        tmp1= tmp1->next= new ele(tmp2->t, tail, tmp1);
};        
/// Destructor      
template<class T>
list<T>::~list()
{
    for(ele *tmp= head->next; tmp != tail; tmp= tmp->next)
        delete tmp->prev;

    delete tail->prev;
    delete tail;
};



/**
 **********************************************************************
 * Zuweisungsoperatoren                                               *
 **********************************************************************
 */

template<class T>
list<T>& list<T>::operator=(const list& l)
{
    if (&l == this)
        return *this;

    eraselist();
    
    ele *tmp1= head;
    for (ele *tmp2= l.head->next; tmp2 != l.tail; tmp2= tmp2->next)
        tmp1= tmp1->next= new ele(tmp2->t, tail, tmp1);

    return *this;
};

template<class T>
list<T>& list<T>::operator+=(const list& l)
{
    for (ele *tmp= l.head->next; tmp != l.tail; tmp= tmp->next)
        insert(tmp->t);

    return *this;
};

template<class T>
list<T>& list<T>::operator+ (const list& l)
{
    list &tmp= *new list(*this);

    tmp+= l;

    return tmp;
};

template<class T>
T&   list<T>::operator[](unsigned long i)
{
  ele *tmp= head->next;
  for (int cnt= 0; tmp != tail; cnt++, tmp= tmp->next)
    if (cnt == i)
      return tmp->t;
  
  errorexit("index out of bounds in list operator[]!");  
  return head->t;
};


/// gibt die neue Anzahl von Eintraegen zurueck.
template<class T>
int list<T>::push_back(const T& t)
{
  ele *tmp= tail;
  ele *tmp1= new ele(t, tmp, tmp->prev);
  tmp->prev->next= tmp1;
  tmp->prev= tmp1;
  return ++n;
};


/// gibt die neue Anzahl von Eintraegen zurueck.
template<class T>
int list<T>::insert(const T& t)
{
  ele *tmp;
  for (tmp= head->next; tmp != tail; tmp= tmp-> next)
    if (tmp->t > t)
      break;
  ele *tmp1= new ele(t, tmp, tmp->prev);
  tmp->prev->next= tmp1;
  tmp->prev= tmp1;
  return ++n;
};


/// gibt die neue Anzahl von Eintraegen zurueck.
template<class T>
int list<T>::remove(const T& t)
{
  ele *tmp;
  for (tmp= head->next; tmp->t != t; tmp= tmp-> next)
    if (tmp == tail)
      return n;

  if (tmp->t == t)
    {  tmp->prev->next= tmp->next;
       tmp->next->prev= tmp->prev;
       delete tmp;
       return --n;
    }
  return n;
};


/// gibt die Position zurueck, an der t gefunden wurde.
template<class T>
int list<T>::search(const T& t)
{
    ele *tmp= head->next;
    for (int i= 1; tmp != tail; i++, tmp= tmp->next)
        if (tmp->t == t)
            return i;

    return 0;
};


/**
 gibt die Position zurueck, an der t gefunden wurde.
 Ausserdem wird eine Referenz auf t in der Liste zurueckgegeben.
 Sinnvoll, wenn die Vergleichsoperatoren nur Schluesselfelder
 beachten und die uebrigen Informationen aus der Liste gezogen werden.
 */
template<class T>
int list<T>::retrieve(T& t)
{
    ele *tmp= head->next;
    for (int i= 1; tmp != tail; i++, tmp= tmp->next)
        if (tmp->t == t)
	  { t= tmp->t;
            return i;
	  };

    return 0;
};

/**
 gibt die Position zurueck, an der t gefunden wurde.
 Ausserdem wird eine Referenz auf t in der Liste zurueckgegeben.
 Sinnvoll, wenn die Vergleichsoperatoren nur Schluesselfelder
 beachten und die uebrigen Informationen aus der Liste gezogen werden.
 */
template<class T>
int list<T>::retrieve(T** t)
{
    ele *tmp= head->next;
    for (int i= 1; tmp != tail; i++, tmp= tmp->next)
        if (tmp->t == **t)
	  { *t= &tmp->t;
            return i;
	  };
    *t= NULL;
    return 0;
};


template<class T>
void list<T>::eraselist()
{
    for(ele *tmp= head->next; tmp != tail; tmp= tmp->next)
        delete tmp->prev;
    delete tail->prev;
    delete tail;

    head= new ele; tail= new ele;
    head->next= tail->next= tail;
    head->prev= tail->prev= head;
};    


/**
 **********************************************************************
 * Vergleichsoperatoren                                               *
 **********************************************************************
 */

template<class T>
int list<T>::operator==(const list& l)
{
    if (n != l.n)
        return 0;

    ele *tmp1= head->next;
    for (ele *tmp2=l.head->next; tmp1 == tail; tmp1= tmp1->next, tmp2= tmp2->next)
        if (tmp1->t != tmp2->t)
            return 0;

    return 1;
};

template<class T>
int list<T>::operator!=(const list& l)
{
    if (n != l.n)
        return 1;

    ele *tmp1= head->next;
    for (ele *tmp2=l.head->next; tmp1 == tail; tmp1= tmp1->next, tmp2= tmp2->next)
        if (tmp1->t != tmp2->t)
            return 1;

    return 0;
};


#endif
