/****************************************************************************
 *                                                                          *
 *  class string                                                            *
 *  ------------                                                            *
 *                                                                          *
 ****************************************************************************/

#include "string.h"
#include <string.h>
#ifdef _SunOS_IRIX_
#include <strings.h> // for bzero
#endif


#ifndef _ERROREXIT_
#define _ERROREXIT_
static int errorexit(char * s)
{
  cerr << s << endl;
  exit(1);
}
#endif

string itoa(long n)
{
  char buf[10];
  sprintf(buf,"%d", n);
  return string(buf);
};

long atoi(const string& s)
{
  return string::atoi(s);
};


/**********************************************************************
 * friends: io-operationen                                            *
 **********************************************************************/

ostream& operator<<(ostream& os, const string& s)
{
    os << s.s;
    return os;
};

istream& operator>>(istream& is, const string& s)
{
    is >> s.s;
    s.checksize();
    return is;
};


string substring(string& dest, string& src, int anfang, int ende= 0)
{
  ende= (ende == 0 || ende > src.len()) ? src.len() : ende;

  if (ende - anfang >= dest.maxlen)
    dest.resize(ende - anfang + 2);

  dest.bzero();
  for (int i= anfang; i < ende; i++)
    dest+= src[i];
  return dest;
};



/**********************************************************************
 * Konstruktoren, Destruktor                                          *
 **********************************************************************/

void string::init(uint siz)
{
  s     = new char[siz+1];
  maxlen= siz;
  l     = 0;

  if (s == NULL)
    errorexit("can\'t allocate memory.");

  for (char *tmp= s; tmp < s+maxlen+1; *(tmp++)= '\0')
    ;
};


string::string(uint siz = 80)
{
  init(siz);
};

string::string(short int siz)
{
  init(siz);
};


string::string(const string& str) : s(new char[str.maxlen+1]),
                                    maxlen(str.maxlen), l(str.l)
{
    if (s == NULL)
        errorexit("can\'t allocate memory.");

    strncpy(s, str.s, (size_t) maxlen);
    s[maxlen]= '\0';
//    cout << "Copy from \"" << str.s << "\" to \"" << s << "\"." << endl;
//    cout << maxlen;
};


string::string(const char *str)
{
    if (str == NULL)
      {s= new char[1];
       s[0]= '\0';
       maxlen= 0;
       l     = 0;
       return;
     }
    s     = new char[strlen(str)+1];
    maxlen= strlen(str);
    l     = strlen(str);
    if (s == NULL)
        errorexit("can\'t allocate memory.");
    strcpy(s, str);
    s[l]= '\0';
};

string::string(const char *str, uint siz)
{
  init(siz);
  if (str != NULL) {
    strncpy(s, str, siz);
    s[siz]= '\0';
  }
};

string::string(const char c)
{
  s     = new char[10];
  maxlen= 10;
  l     = 1;
  if (s == NULL)
    errorexit("can\'t allocate memory.");
  s[0]= c;
  s[1]= '\0';
};



string::~string()
{
  delete [] s;
};

void string::checksize() const
{
  char *tmp;
  for (tmp= s; (*tmp !=  '\0') && (tmp < s + maxlen); tmp++)
    ;

  if (tmp == s + maxlen)
    { *tmp= '\0';
      l= maxlen;
    }
  else
    l= tmp - s;
};

void string::resize(uint siz)
{
    char *tmp= new char[siz+1];
    if (tmp == NULL)
        errorexit("can\'t allocate memory.");

    maxlen= siz;
    l= (siz < l) ? siz : l;

    strncpy(tmp, s, maxlen);
    delete [] s;
    s= tmp;
};

void string::bzero()
{
  ::bzero(s, maxlen);
  l= 0;
};


void string::upcase()
{
  for (int i= 0; i < l; i++)
    s[i]= toupper(s[i]);
};


void string::downcase()
{
  for (int i= 0; i < l; i++)
    s[i]= tolower(s[i]);
};


/**********************************************************************
 * Zuweisungsoperatoren                                               *
 **********************************************************************/

string& string::operator=(const string& str)
{
    if (&str == this)
        return *this;

    delete[] s;

    s     = new char[str.maxlen+1];
    maxlen= str.maxlen;
    l     = str.l;

    strncpy(s, str.s, maxlen);
    s[maxlen]= '\0';
    return *this;
};


string& string::operator=(const char *str)
{
    if (str == s)
        return *this;

    delete[] s;

    s     = new char[strlen(str)+1];
    maxlen= strlen(str);
    l     = strlen(str);
    strcpy(s, str);
    s[l]  = '\0';

    return *this;
};



/**********************************************************************
 * Vergleichsoperatoren                                               *
 **********************************************************************/

int string::operator==(const string& str) const
{
  return (strcmp(s, str.s) == 0);
};

int string::operator!=(const string& str) const
{
    return (strcmp(s, str.s) != 0);
};

int string::operator<=(const string& str) const
{
    return (strcmp(s, str.s) <= 0);
};

int string::operator>=(const string& str) const
{
    return (strcmp(s, str.s) >= 0);
};

int string::operator< (const string& str) const
{
    return (strcmp(s, str.s) < 0);
};

int string::operator> (const string& str) const
{
    return (strcmp(s, str.s) > 0);
};

int string::contains(const char c) const
{
  for (uint i= 0; i < len(); i++)
    if (c == s[i])
      return 1;

  return 0;
};


int string::find(string sub)
{
  char *p=strstr(s, sub.s);

  if (p == NULL)
    return -1;

  return p-s;
}





//int strcmp(const string& s1, const string& s2)  // friend
//{
//    return strcmp((char *) s1, (char *) s2);
//};



/**********************************************************************
 * Addition, Subtraktion, ...                                         *
 **********************************************************************/

string string::operator +(const string& str)
{
  string tmp(l + str.l);

  char *t1= tmp.s;
  char *t2;
  for ( t2= s; *t2 != '\0'; *(t1++)= *(t2++) )
    ;

  for ( t2= str.s  ; *t2 != '\0'; *(t1++)= *(t2++) )
    ;

  *t1= '\0';
  tmp.l= strlen(tmp.s);

  return tmp;
};


string& string::operator -(const string& str)
{

    return *this;
};


string& string::operator +=(const string& str)
{
    char *tmp= s;
    if (maxlen < l + str.l + 1)
      { tmp= new char[l + str.l + 10];
        strcpy(tmp, s);
        maxlen= l + str.l + 1;
        delete [] s;
        s= tmp;
      }

    strcat(tmp, str.s);
    l+= str.l;
    s[l]='\0';
    return *this;
};


string& string::operator -=(const string& str)
{
    return *this;
};



/**********************************************************************
 * Indexoperatoren                                                    *
 **********************************************************************/


//  const char& string::operator[](uint i) const
//  {
//    if (i > maxlen) {
//      // cout << "string= \"" << s << "\" i= " << i << endl;
//      errorexit("Index out of Array.");
//    }
//
//      return s[i];
//  };



//  char& string::operator[](uint i)
//  {
//    if (i > maxlen) {
//      cout << "string= \"" << s << "\" i= " << i << "maxlen: "
//           << maxlen << endl;
//      errorexit("Index out of Array.");
//    }
//
//      return s[i];
//  };



/**********************************************************************
 * Cast-Funktionen                                                    *
 **********************************************************************/

string& string::fromul(unsigned long n)
{
    sprintf(s, "%u", n);
    l= strlen(s);
    if (l > maxlen)
        errorexit("String zu klein, um Daten aufzunehmen.");
    return *this;
};


string& string::fromint(long n)
{
    sprintf(s, "%d", n);
    l= strlen(s);
    if (l > maxlen)
        errorexit("String zu klein, um Daten aufzunehmen.");
    return *this;
};

string& string::fromdouble(double f)
{
    sprintf(s, "%f", f);
    l= strlen(s);
    if (l > maxlen)
        errorexit("String zu klein, um Daten aufzunehmen.");
    return *this;
};

string& string::fromchar(char c)
{
    if (maxlen < 1)
        errorexit("String zu klein, um Daten aufzunehmen.");

    s[0]= c;
    s[1]= '\0';
    l= 1;
    return *this;
};

// static
string& string::itoa(long n)
{
  string& s= *new string();
  s.fromint(n);
  return s;
};

// static
string& string::ultoa(unsigned long n)
{
  string& s= *new string();
  s.fromul(n);
  return s;
};


//static
unsigned long   string::atoul(const char *s)
{
  unsigned long ul= 0;
  const char *pch= s;

  for (int i=1; *pch != '\0'; pch++, i++) {
    ul*=10;
    ul+=*pch - '0';
  }

  return ul;
};


