// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#ifdef WIN32
#include <stdlib.h>
#else
#include <strstream>
#endif

#include <iostream>
#include "strtools.h"

using std::string;

#ifdef WIN32
string ltos(const long i) {
  static char buf[64];
  return string(ltoa(i,buf,10));
}

string itos(const int  i) {
  static char buf[64];
  return string(itoa(i,buf,10));
}
#else
string ltos(const long i) {
  char       *buf;
  string      s;
  std::ostrstream  os;

  os << i;
  os << '\x0' << std::ends;

  buf= os.str();
  s= buf;
  delete[] buf;
  return s;
}

string itos(const int  i) {
  char       *buf;
  string      s;
  std::ostrstream  os;

  os << i << std::ends;

  buf= os.str();
  s= buf;
  delete[] buf;
  return s;
}
#endif


string upcase(const string& s_in) {
  string s = s_in;
  for (string::size_type i = 0; i < s_in.size(); i++) {
    if ((s[i] >= 'a') && (s[i] <= 'z')) {
      s[i] -= ' ';
    }
  }
  return s;
}

string downcase(const string& s_in) {
  string s = s_in;
  for (string::size_type i = 0; i < s_in.size(); i++) {
    if ((s[i] >= 'A') && (s[i] <= 'Z')) {
      s[i] += ' ';
    }
  }
  return s;
}

string gsub(string& s, const string& srch, const string& repl) {
  string::size_type i = 0;
  while ((i < s.size()) && ((i = s.find(srch, i)) != string::npos)) {
    s.replace(i, srch.size(), repl);
    i += repl.size();
  }
  return s;
}

string before(const string& s_in, const string& srch) {
  if (s_in.length() < srch.length())
    return "";
  string::size_type i = s_in.find(srch);
  return ((i != string::npos) ? s_in.substr(0,i) : "");
}

string before(const string& s_in, const string::size_type i) {
  return s_in.substr(0,i);
}

string after(const string& s_in, const string& srch) {
  if (s_in.length() < srch.length())
    return "";
  string::size_type i = s_in.find(srch);
  return ((i != string::npos) ? s_in.substr(i+srch.size()) : "");
}

string after(const string& s_in, const string::size_type i) {
  return s_in.substr(i+1);
}

string from(const string& s_in, const string& srch) {
  string::size_type i = s_in.find(srch);
  return ((i != string::npos) ? s_in.substr(i) : NULL);
}

string from(const string& s_in, const string::size_type i) {
  return s_in.substr(i);
}

int split(const string& s, string w[], int anz, string sep) {
  // find words separated by sep
  int wc = 0;
  string::size_type i_b = 0;
  string::size_type i_e = 0;
  while ((wc < anz) && ((i_e = s.find_first_of(sep, i_b)) != string::npos)) {
    w[wc++] = s.substr(i_b, (i_e - i_b));
    i_b = i_e + 1;
  }
  // copy remaining string
  if (wc < anz)
    w[wc++] = s.substr(i_b);
  return wc;
}


