// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#include <util/tcltimer.h>

#ifdef WIN32
using std::list<TclTimerToken*>;
#endif

TclTimer::TclTimer(string i, P2TimerInterface p) {
  id=i;
  pif=p;
};

TclTimer::~TclTimer() {
  cancel();
};

void TclTimer::cancel() {
  TclTokenList::iterator pos;
  for (pos = token_list.begin(); pos != token_list.end(); ++pos)
    if ((*pos)->token != NULL) {
      Tcl_DeleteTimerHandler((*pos)->token);
      delete *pos;
    }
  token_list.clear();
};

void TclTimer::addToken(int msecs) {
  static TclTimerToken* tt;
  tt = new TclTimerToken();
  tt->cd = (ClientData)this;
  token_list.push_back(tt);
  TclTokenList::iterator pos = token_list.end();
  --pos;
  (*pos)->index = pos;
  (*pos)->token = Tcl_CreateTimerHandler(msecs, dispatch, &(**pos));
};

void TclTimer::dispatch(ClientData cd) {
  P2TclTimerToken ptoken = (P2TclTimerToken) cd;
  P2TclTimer ptimer = (P2TclTimer) ptoken->cd;
  ptimer->token_list.erase(ptoken->index);
  delete ptoken;
  if (ptimer->getPif())
    ptimer->getPif()->timerHandler(ptimer->getId());
};

