// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#ifndef _TCL_TIMER_H_
#define _TCL_TIMER_H_

#ifdef WIN32
#  pragma warning (disable: 4786)
#endif


#ifdef WIN32
#include "win32.h"
#else
#include <sys/time.h>
#endif

#include <string>
#include <list>
#include <iterator>
#include <tcl.h>

#include <util/timerinterface.h>

class TclTimerToken;

#ifdef WIN32
using std::list<TclTimerToken*>;
#endif


class TclTimerToken {
 public:
  Tcl_TimerToken token;
  ClientData	 cd;
  std::list<TclTimerToken*>::iterator index;
};

typedef TclTimerToken *P2TclTimerToken;
typedef std::list<TclTimerToken*> TclTokenList;


class TclTimer : public Timer {

public:
  TclTimer(string ="noname", P2TimerInterface =NULL);
  ~TclTimer();

  string getId() { return id; };
  void setId(string i) { id=i; };

  P2TimerInterface getPif() { return pif; };
  void setPif(P2TimerInterface p) { pif=p; };

  void msched(int msecs) { addToken(msecs); };
  void usched(int usecs) { addToken(usecs/1000); };
  void ssched(int secs)  { addToken(secs*1000); };

  int isActive() { return(!token_list.empty()); };
  void cancel();

private:
  P2TimerInterface	pif;
  string		id;
  TclTokenList	        token_list;

  void addToken(int);
  static void dispatch(ClientData);

};
typedef TclTimer *P2TclTimer;

#endif







