// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#ifndef _RTPLIB_TOOLS_H_
#define _RTPLIB_TOOLS_H_

#ifdef WIN32
#include <winsock.h>
#include "win32.h"
#else
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/time.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

#include <string>

#include <util/types.h>


// defines considering timeval
// macro comparing allowing equal values
#define  timercmp_eq(tvp, uvp, cmp) ((tvp).tv_sec cmp (uvp).tv_sec || ((tvp).tv_sec == (uvp).tv_sec && ((tvp).tv_usec cmp (uvp).tv_usec || (tvp).tv_usec == (uvp).tv_usec)))
#define TIMEVAL_MAX gen_timeval(LONG_MAX, LONG_MAX)


/* Offset from UNIX's epoch to the NTP epoch in seconds (NTP's JAN_1970) */
#define RTP_EPOCH_OFFSET	2208988800UL


u_int32	get_local_addr(int =0);
u_int32	get_local_host();

inline int double2int(double x) {
  x += .5;
  return (int)x;
}

// some useful rtp and ntp utility routines

inline timeval unixtime()
{
	timeval tv;
	gettimeofday(&tv, NULL);
	return (tv);
}

inline u_int32 ntp2unix(u_int32 ntp)
{
  if(ntp==0) return(0);
  return(ntp-0x83aa7e80);
};

inline u_int32 unix2ntp(u_int32 time)
{
  if(time==0) return(0);
  return(time+0x83aa7e80);
};

inline u_int32 sdptime()
{
  timeval tv = unixtime();
  return unix2ntp((u_int32) tv.tv_sec);
};

inline uint32 rtpitime()
{
  timeval tv = unixtime();                    // based on 1.1.1970
  uint32 ms = unix2ntp((u_int32) tv.tv_sec);  // convert to base 1.1.1900
  ms *= 1000;                                 // sec -> milliseconds
  uint32 ms_2 = tv.tv_usec / 1000;            // microsec -> millisec
  return ms + ms_2;
};

inline ntp64 ntp64time(const timeval& tv)
{
	ntp64 n;
	n.upper = tv.tv_sec + RTP_EPOCH_OFFSET;

	// convert microseconds to fraction of second * 2^32 (i.e., the lsw of
        // a 64-bit ntp timestamp).  This routine uses the factorization
        // 2^32/10^6 = 4096 + 256 - 1825/32 which results in a max conversion
        // error of 3 * 10^-7 and an average error of half that.
	n.lower = (tv.tv_usec * 1825) >> 5;
	n.lower = (tv.tv_usec << 12) + (tv.tv_usec << 8) - n.lower;
	return (n);
}

inline timeval ntp2tv(const ntp64& ntp)
{
  timeval tv;
  tv.tv_sec = (int) (ntp.upper - RTP_EPOCH_OFFSET);

  // factorization: 10^6/2^32 = 2^-12 - 2^-16 + 2^-18 + 2^-23 + 2^-26
  u_int32 usec = ntp.lower;
  usec = ((usec>>12) - (usec>>16) + (usec>>18) + (usec>>23)) + (usec>>26);
  tv.tv_usec=(int) usec;
  return (tv);

};

inline timeval tvdiff(const timeval& tv1, const timeval& tv2)
{
  timeval tv;
  tv.tv_sec = tv1.tv_sec - tv2.tv_sec - (tv1.tv_usec<tv2.tv_usec?1:0);
  tv.tv_usec= (tv1.tv_usec<tv2.tv_usec)?(1000000-tv2.tv_usec+tv1.tv_usec):(tv1.tv_usec-tv2.tv_usec);
  return (tv);
};

inline timeval gen_timeval(const long s, const long us) {
  timeval tv;
  tv.tv_sec = s;
  tv.tv_usec = us;
  return tv;
}


inline uint32 tvtoms(const timeval& tv)
{
  uint32 ms = (uint32)tv.tv_sec * 1000;   // sec -> millisec
  ms += (uint32)tv.tv_usec / 1000;        // microsec -> millisec
  return ms;
};

inline timeval mstotv(const uint32 ms)
{
  timeval tv;
  tv.tv_sec = (ms / 1000);        // millisec -> sec
  tv.tv_usec = (ms % 1000) * 1000;         // remainder -> microsec
  return tv;
};

inline timeval tvsubms(const timeval& t, const uint32 ms)
{
  long sec = ms / 1000;
  long usec = (ms % 1000) * 1000;
  timeval tv;
  tv.tv_sec = t.tv_sec - sec - (t.tv_usec < usec ? 1 : 0);
  tv.tv_usec = (t.tv_usec < usec) ? (1000000 - usec + t.tv_usec) : (t.tv_usec - usec);
  return tv;
};

inline timeval tvaddms(const timeval& t, const uint32 ms)
{
  long sec = ms / 1000;
  long usec = (ms % 1000) * 1000;
  timeval tv;
  tv.tv_sec = t.tv_sec + sec + ((usec + t.tv_usec) / 1000000);
  tv.tv_usec = ((usec + t.tv_usec) % 1000000);
  return tv;
};

inline bool operator<(const timeval& tv1, const timeval& tv2) {
  return timercmp(&tv1, &tv2, <);
}
inline bool operator>(const timeval& tv1, const timeval& tv2) {
  return timercmp(&tv1, &tv2, >);
}
inline bool operator>=(const timeval& tv1, const timeval& tv2) {
  return timercmp_eq(tv1, tv2, >);
}
inline bool operator<=(const timeval& tv1, const timeval& tv2) {
  return timercmp_eq(tv1, tv2, <);
}
inline bool operator==(const timeval& tv1, const timeval& tv2) {
  return ((tv1.tv_sec == tv2.tv_sec) && (tv1.tv_usec == tv2.tv_usec));
}
inline bool operator!=(const timeval& tv1, const timeval& tv2) {
  return ((tv1.tv_sec != tv2.tv_sec) || (tv1.tv_usec != tv2.tv_usec));
}

#endif

