// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#ifndef TYPES_H
#define TYPES_H

#ifdef WIN32
#include <winsock.h>
#include <util/win32.h>
#else
#include <arpa/inet.h>
#endif

#include <iostream>
#include <sys/types.h>
 
///  8 bit signed integer
typedef char                int8;
///  8 bit unsigned integer
typedef unsigned char       uint8;
/// 16 bit signed integer
typedef short int           int16;
/// 16 bit unsigned integer
typedef unsigned short int  uint16;
/// 32 bit signed integer
typedef int                 int32;
/// 32 bit unsigned integer
typedef unsigned int        uint32;
/// 32 bit floating point number
typedef float               ieee32;
/// 64 bit floating point number
typedef double              ieee64;

// already defined in sys/types.h
//typedef unsigned long       ulong;

///  8 bit unsigned integer
typedef unsigned char u_int8;
/// 16 bit unsigned integer
typedef unsigned short u_int16;
/// 32 bit unsigned integer
typedef unsigned int u_int32;

// extra types

/*
// 64 bit unsigned integer
class uint64 {
 public:
  // friend ostream& operator<<(ostream&, const uint64&);

  uint64() : high(0), low(0) {}
  uint64(uint32 _low) : high(0), low(_low) {}
  uint64(uint32 _high, uint32 _low) : high(_high), low(_low) {}
  uint64(const uint64& u) : high(u.high), low(u.low) {}

  uint64& operator=(const uint64& u) {
    if (this == &u) return *this;
    high = u.high;
    low = u.low;
    return *this;
  }

  bool operator==(const uint64& i) const {
    return ((high == i.high) && (low == i.low));
  }
  bool operator!=(const uint64& i) const {
    return ((high != i.high) || (low != i.low));
  }
  bool operator<(const uint64& i) const {
    return ((high < i.high) || ((high == i.high) && (low < i.low)));
  }
  bool operator>(const uint64& i) const {
    return ((high > i.high) || ((high == i.high) && (low > i.low)));
  }

  uint32 high;
  uint32 low;
};
*/

/*
 * Handle 64 bit ntp timestamps
 */
typedef struct {
	u_int32 upper;	/* more significant 32 bits */
	u_int32 lower;	/* less significant 32 bits */
} ntp64;


extern bool cmp_wrapped_ts_smaller(const uint32&, const uint32&);
extern bool cmp_wrapped_ts_bigger(const uint32&, const uint32&);
extern bool cmp_wrapped_ts_sml_eq(const uint32&, const uint32&);
extern bool cmp_wrapped_ts_bg_eq(const uint32&, const uint32&);

/*
#ifdef WIN32
extern ::ostream& operator<<(::ostream&, const uint64&);
#else
extern ostream& operator<<(ostream&, const uint64&);
#endif
*/

#endif // TYPES_H


