// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Christoph Kuhmuench, Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {cjk,xshilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#include "util.h"


int Debug::debug= 0;

void error_sys(std::string s)
  // {{{ body
{
  std::cerr << s << std::endl;
  std::cerr << "errorno:  " << errno << std::endl;
  std::cerr << "errormsg: " << strerror(errno) << std::endl;
};
// }}}


void error_quit(std::string s)
  // {{{ body
{
  std::cerr << s << std::endl;
  std::cerr << "errorno:  " << errno << std::endl;
  std::cerr << "errormsg: " << strerror(errno) << std::endl;
  exit(-1);
};
// }}}


//friend
std::ostream& operator<<(std::ostream& os, const Exception &e)
  // {{{ body
{
  os << e.toString();
  return os;
};
// }}}


Exception::Exception()
{};


char    *SystemException::systemErrStrs[] = { "ok",
                                              "Out of Memory",
                                              "Array Index out of bounds" };

//virtual
SystemException::~SystemException()
{};


//virtual
string SystemException::toString() const
{
  char buf[10];
  sprintf(buf,"%d", -systemErrVal);
  string s= string("System Error No. ") + string(buf) +
            string(": ") + string(systemErrStrs[-systemErrVal]) +
            string("\t[") + string(systemErrParam) + string("].\n");
  return s;
};

