// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
// Copyright (c) 2000 University of Mannheim. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for educational, research and non-profit purposes,
// without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice, this paragraph and the
// following four paragraphs appear in all copies.
//
// Permission to incorporate this software into commercial products may
// be obtained from the following address:
// University of Mannheim
// Lehrstuhl fuer Praktische Informatik IV
// L15, 16
// 68161 Mannheim, Germany
//
// IN NO EVENT SHALL THE UNIVERSITY OF MANNHEIM BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
// ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF MANNHEIM HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE UNIVERSITY OF MANNHEIM SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
// IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF MANNHEIM HAS NO OBLIGATIONS
// TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The University of Mannheim will appreciate receiving any enhancements and
// modifications of this software made by others, and reserves the right to
// include them in future releases. The University also recommends that this
// software not be redistributed by third parties, but be obtained by all
// directly from the University's server, so that all users of the software
// may receive bug notifications and other relevant announcements.
//
// Authors:
//   Christoph Kuhmuench, Volker Hilt, Juergen Vogel, Martin Mauve
//   University of Mannheim
//   {cjk,hilt,vogel,mauve}@informatik.uni-mannheim.de
//
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#ifndef _UTIL_H_
#define _UTIL_H_

#ifdef WIN32
#include <util/win32.h>
#endif

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string>

using std::string;

//#define DEBUG 1

#define boolean int
#define true 1
#define false 0

#define DEBUG_OFF 0
#define DEBUG_NET 1
#define DEBUG_SDP 2
#define DEBUG_CLIENT 4
#define DEBUG_SERVER 16
#define DEBUG_OTHER 32768



void error_sys(string s);
void error_quit(string s);



class Exception
// {{{ definition
{
  friend std::ostream& operator<<(std::ostream& os, const Exception &e);

public:
  Exception();

  virtual std::string toString() const { return std::string(""); };

  //  operator const char* () const  { return toString().c_str(); };
};
// }}}


typedef enum
{
  SYSTEM_ERR_OK                    =  0,
  SYSTEM_ERR_OUT_OF_MEMORY         = -1,
  SYSTEM_ARRAY_INDEX_OUT_OF_BOUNDS = -2
} system_error_t;


class SystemException
// {{{ definition
{
  friend std::ostream& operator<<(std::ostream& os, const Exception &e);

private:
  static char *systemErrStrs[];
  int          systemErrVal;
  std::string       systemErrParam;

public:
  SystemException(system_error_t _systemErrVal, string _systemErrParam)
    : systemErrVal(_systemErrVal), systemErrParam(_systemErrParam)
    {};

  virtual ~SystemException();


  virtual std::string toString() const;
};
// }}}



class Debug
// {{{ definition
{
public:
  static int debug;

public:
  inline static void out(int t, std::string s)
    { if (debug & t) std::cout << "[Debug: " << t << ", " << debug << "] " << s.c_str() << std::endl; }
};
// }}}

#endif


